

local LootLink = LootLink;




function LootLink:Compatibility_Initialize()
	ItemLinks = self.Database;
	
	
	LootLinkState = {};
	
	setmetatable(LootLinkState, self.LOOTLINK_STATE_METATABLE);
	
	
	LootLinkTooltip = self.RESULTS_TOOLTIP;
	
	
	self.Tooltips_AddSellPrice = self.Tooltips_AddVendorValue;
	
	
	if (self.ReplaceHook_AtlasLootTooltip_SetHyperlink) then
		self.Original_AtlasLootTooltip_SetHyperlink = AtlasLootTooltip.SetHyperlink;
		AtlasLootTooltip.SetHyperlink = self.ReplaceHook_AtlasLootTooltip_SetHyperlink;
	end
	
	
	if (EquipCompare_RegisterTooltip) then
		EquipCompare_RegisterTooltip(self.RESULTS_TOOLTIP);
	end
end


LootLink.AddTooltipSellPrice = (function(baseID, tooltip, quantity) LootLink:Tooltips_AddVendorValue(baseID, tooltip, quantity); end);




function LootLink:Compatibility_GetDatabaseIndex(key)
	if (type(key) ~= "string") then return nil; end
	
	
	local itemTable, suffixID = self:Matching_GetItemTableAndSuffixIDByName(self.Database, key);
	
	
	if (suffixID) then
		return nil;
	else
		return itemTable;
	end
end



function LootLink:Compatibility_GetLootLinkStateIndex(key)
	if (key == "ServerNamesToIndices") then
		return self.Config.LastSeenOnServer;
	end
	
	return nil;
end




function LootLink:Compatibility_GetBaseTableIndex(itemTable, key)
	if (type(key) ~= "string") then
		return rawget(itemTable, key);
	end
	
	
	key = key:lower();
	
	
	if (key == "baseid" or key == "id") then
		return rawget(itemTable, -1);
		
	elseif (key == "name" or key == "na") then
		return rawget(itemTable, 1);
		
	elseif (key == "color" or key == "co") then
		return rawget(itemTable, 2);
		
	elseif (key == "suffixes" or key == "su") then
		return rawget(itemTable, 3);
		
	elseif (key == "bind" or key == "bi") then
		return rawget(itemTable, 4);
		
	elseif (key == "text" or key == "te") then
		return rawget(itemTable, 5);
		
	elseif (key == "price" or key == "pr") then
		return rawget(itemTable, 6);
		
	elseif (key == "location" or key == "lo") then
		return rawget(itemTable, 7);
		
	elseif (key == "type" or key == "ty") then
		return rawget(itemTable, 8);
		
	elseif (key == "subtype" or key == "su") then
		return rawget(itemTable, 9);
		
	elseif (key == "equiplevel" or key == "eq") then
		return rawget(itemTable, 10);
		
	elseif (key == "unique" or key == "un") then
		return rawget(itemTable, 11);
		
	elseif (key == "itemlevel" or key == "lv") then
		return rawget(itemTable, 12);
		
	elseif (key == "texture" or key == "tx") then
		return rawget(itemTable, 13);
		
		
	-- The rest are for compatibility with the old LootLink's database format.
		
	elseif (key == "i") then
		return (rawget(itemTable, -1)..":0:0:0:0:0:0:0");
		
	elseif (key == "c") then
		local rarityTable = ITEM_QUALITY_COLORS[rawget(itemTable, 2)];
		
		return ((not rarityTable and "ff40ffc0") or rarityTable.hex:sub(3)); -- Trim the "|c" from "|cffffffff".
		
	elseif (key == "p") then
		return rawget(itemTable, 6);
		
	elseif (key == "t") then
		return self:ItemTables_GetConcatenatedFullText(self, nil);
	end
	
	
	return rawget(itemTable, key);
end



function LootLink:Compatibility_SetBaseTableIndex(itemTable, key, value)
	if (type(key) ~= "string") then
		rawset(itemTable, key, value);
		return;
	end
	
	
	key = key:lower();
	
	
	if (key == "baseid" or key == "id") then
		rawset(itemTable, -1, value);
		
	elseif (key == "name" or key == "na") then
		rawset(itemTable, 1, value);
		
	elseif (key == "color" or key == "co") then
		rawset(itemTable, 2, value);
		
	elseif (key == "suffixes" or key == "su") then
		rawset(itemTable, 3, value);
		
	elseif (key == "bind" or key == "bi") then
		rawset(itemTable, 4, value);
		
	elseif (key == "text" or key == "te") then
		rawset(itemTable, 5, value);
		
	elseif (key == "price" or key == "pr") then
		rawset(itemTable, 6, value);
		
	elseif (key == "location" or key == "lo") then
		rawset(itemTable, 7, value);
		
	elseif (key == "type" or key == "ty") then
		rawset(itemTable, 8, value);
		
	elseif (key == "subtype" or key == "su") then
		rawset(itemTable, 9, value);
		
	elseif (key == "equiplevel" or key == "eq") then
		rawset(itemTable, 10, value);
		
	elseif (key == "unique" or key == "un") then
		rawset(itemTable, 11, value);
		
	elseif (key == "itemlevel" or key == "lv") then
		rawset(itemTable, 12, value);
		
	elseif (key == "texture" or key == "tx") then
		rawset(itemTable, 13, value);
		
		
	-- The rest are for compatibility with the old LootLink's database format.
		
	elseif (key == "c") then
		rawset(itemTable, 2, self:Parsing_GetRarityNumberForHex(value));
		
	elseif (key == "p") then
		rawset(itemTable, 6, value);
		
	elseif (key ~= "i" and key ~= "t" and key ~= "d") then
		rawset(itemTable, key, value);
	end
end




function LootLink:Compatibility_GetSuffixTableIndex(itemTable, key)
	if (type(key) ~= "string") then
		return rawget(itemTable, key);
	end
	
	
	key = key:lower();
	
	
	if (key == "specid" or key == "id") then
		return rawget(itemTable, 1);
		
	elseif (key == "text" or key == "te") then
		return rawget(itemTable, 2);
	end
	
	
	return rawget(itemTable, key);
end



function LootLink:Compatibility_SetSuffixTableIndex(itemTable, key, value)
	if (type(key) ~= "string") then
		rawset(itemTable, key, value);
		return;
	end
	
	
	key = key:lower();
	
	
	if (key == "specid" or key == "id") then
		rawset(itemTable, 1, value);
		
	elseif (key == "text" or key == "te") then
		rawset(itemTable, 2, value);
		
	else
		rawset(itemTable, key, value);
	end
end




function LootLink_AddTooltipInfo(name, tooltip, quantity)
	local itemTable, suffixID = LootLink:Matching_GetItemTableAndSuffixByNameID(LootLink.Database, name);
	
	if (itemTable) then
		LootLink:Tooltips_AddAllApplicableInfo(
		  ("item:"..itemTable[-1]..":0:0:0:0:0:"..(suffixID or 0)..":"..((suffixID and suffixID < 0 and next(itemTable[3][suffixID])) or 0)),
		  tooltip,
		  quantity
		);
	end
end



function LootLink_SetTooltip(tooltip, name, quantity)
	local itemTable, suffixID = LootLink:Matching_GetItemTableAndSuffixIDByName(LootLink.Database, name);
	
	if (itemTable) then
		LootLink:Results_PopulateFakeItemTooltip(tooltip, itemTable[-1], suffixID);
		
		LootLink:Tooltips_AddAllApplicableInfo(
		  ("item:"..itemTable[-1]..":0:0:0:0:0:"..(suffixID or 0)..":"..((suffixID and suffixID < 0 and next(itemTable[3][suffixID])) or 0)),
		  tooltip,
		  quantity
		);
	end
end




function LootLink_AddItem(name, link, color)
	local baseID, suffixID = link:match("(%d+):%-?%d+:%-?%d+:%%-?d+:%-?%d+:%-?%d+:(%-?%d+):%-?%d+");
	
	if (not baseID) then
		baseID, suffixID = link:match("(%-?%d+):%-?%d+:(%-?%d+):%-?%d+");
	end
	
	baseID = tonumber(baseID);
	if (not baseID) then return nil; end
	
	suffixID = tonumber(suffixID);
	
	
	local baseName, suffix;
	
	if (suffixID ~= 0) then
		baseName, suffix = name:match(LootLink.LOCALIZATION.SUFFIX_SEPARATOR);
	end
	
	
	local stringsub = LootLink.STRING_LIBRARY.sub;
	
	for index, colorTable in pairs(ITEM_QUALITY_COLORS) do
		if (stringsub(colorTable.hex, 3) == color) then
			color = index;
			break;
		end
	end
	
	
	local itemTable = LootLink:ItemTables_AddBaseItem(baseID, (baseName or name), color);
	
	if (itemTable and suffix) then
		LootLink:ItemTables_AddSuffixItem(itemTable, suffix, suffixID);
	end
	
	
	return itemTable;
end




if (IsAddOnLoaded("AtlasLoot") and GetAddOnMetadata("AtlasLoot", "Version") == "2.01.00") then
	
	function LootLink.ReplaceHook_AtlasLootTooltip_SetHyperlink(self, hyperlink)
		LootLink.Original_AtlasLootTooltip_SetHyperlink(self, hyperlink);
		
		LootLink:Tooltips_AddAllApplicableInfo(hyperlink, self, 1);
	end
	
end

