

local LootLink = LootLink;


local localization = LootLink.LOCALIZATION;




LootLink.VERSION = 829;




LootLink.GLOBAL_ENVIRONMENT = getfenv(0); -- This is used in place of calls to "getglobal".




LootLink.TABLE_LIBRARY = table;



LootLink.STRING_LIBRARY = string;

LootLink.STRING_LIBRARY.concat = strconcat;



LootLink.MATH_LIBRARY = math;




LootLink.PLAYER_NAME = UnitName("player");




do
	local frame = CreateFrame("Frame", "LootLink_ScriptsFrame", WorldFrame);
	
	frame:Hide();
	frame.LOOTLINK = LootLink;
	LootLink.SCRIPTS_FRAME = frame;
	
	
	frame = LootLink_ResultsFrame;
	
	frame.LOOTLINK = LootLink;
	LootLink.RESULTS_FRAME = frame;
	
	
	frame = LootLink_SearchFrame;
	
	frame.LOOTLINK = LootLink;
	LootLink.SEARCH_FRAME = frame;
	
	
	frame = LootLink_ResultsTooltip;
	
	frame.LOOTLINK = LootLink;
	LootLink.RESULTS_TOOLTIP = frame;
end




LootLink.NUM_RESULT_BUTTONS = 23;


LootLink.RESULT_BUTTON_HEIGHT = 32;



do
	local createFrame = CreateFrame;
	local parent = LootLink.RESULTS_FRAME;
	
	local button = LootLink_ResultsFrame_ItemButton1;
	
	
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	button.LOOTLINK = LootLink;
	LootLink.RESULTS_BUTTON_1 = button;
	
	for index = 2, LootLink.NUM_RESULT_BUTTONS do
		button = createFrame("Button", "LootLink_ResultsFrame_ItemButton"..index, parent, "LootLink_ResultsFrame_ItemButtonTemplate");
		
		button:SetID(index);
		button:SetPoint("TOPLEFT", "LootLink_ResultsFrame_ItemButton"..(index - 1), "BOTTOMLEFT", 0, 1);
		
		button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
		
		button.LOOTLINK = LootLink;
		LootLink["RESULTS_BUTTON_"..index] = button;
	end
end




LootLink.MAX_RECENTLY_ADDED_ITEMS = 20;




LootLink.MAX_COMMAND_LINE_SEARCH_MATCHES = 10;




do
	local tooltip = CreateFrame("GameTooltip", "LootLink_ParsingTooltip", nil, "GameTooltipTemplate");
	
	
	tooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
	
	tooltip.LOOTLINK = LootLink;
	LootLink.PARSING_TOOLTIP = tooltip;
	
	
	tooltip.LINE_POINTERS = {};
	
	
	local pointersTable = tooltip.LINE_POINTERS;
	local globalEnv = LootLink.GLOBAL_ENVIRONMENT;
	
	for index = 1, 8 do
		pointersTable[index] = globalEnv["LootLink_ParsingTooltipTextLeft"..index];
	end
end




LootLink.MOUSEOVER_PARSING_DELAY = 5;




LootLink.SORTING_LIST = {
	{ name = localization.SORT_NAME },
	{ name = localization.SORT_RARITY },
	{ name = localization.SORT_EQUIPLEVEL, comparisonKey = 10 },
	{ name = localization.SORT_ITEMLEVEL, comparisonKey = 12 },
	{ name = localization.SORT_TYPE, comparisonKey = 8 },
	{ name = localization.SORT_SUBTYPE, comparisonKey = 9 },
	{ name = localization.SORT_EQUIPLOCATION, comparisonKey = 7 },
	{ name = localization.SORT_BASEID },
};




LootLink.BIND_DATA = {
	{ name = localization.BIND_EQUIP, parseName = "ITEM_BIND_ON_EQUIP" },
	{ name = localization.BIND_PICKUP, parseName = "ITEM_BIND_ON_PICKUP" },
	{ name = localization.BIND_USE, parseName = "ITEM_BIND_ON_USE" },
	{ name = ITEM_BIND_QUEST, parseName = "ITEM_BIND_QUEST" },
};




LootLink.EQUIP_LOCATIONS = {
	-- 3 entries
	{ name = ARMOR,
		subList = {
			-- 9 entries
			{ name = INVTYPE_CHEST, parseName1 = "INVTYPE_CHEST",
			  parseName2 = "INVTYPE_ROBE",
			  value = 5
			},
			{ name = INVTYPE_FEET, parseName1 = "INVTYPE_FEET", value = 8 },
			{ name = INVTYPE_HAND, parseName1 = "INVTYPE_HAND", value = 10 },
			{ name = INVTYPE_HEAD, parseName1 = "INVTYPE_HEAD", value = 1 },
			{ name = INVTYPE_LEGS, parseName1 = "INVTYPE_LEGS", value = 7 },
			{ name = localization.DISPLAY_SHIELDS, parseName1 = "INVTYPE_SHIELD", value = 20 },
			{ name = INVTYPE_SHOULDER, parseName1 = "INVTYPE_SHOULDER", value = 3 },
			{ name = INVTYPE_WAIST, parseName1 = "INVTYPE_WAIST", value = 6 },
			{ name = INVTYPE_WRIST, parseName1 = "INVTYPE_WRIST", value = 9 },
		},
	},
	{ name = localization.DISPLAY_WEAPONS,
		subList = {
			-- 5 entries
			{ name = INVTYPE_WEAPON, parseName1 = "INVTYPE_WEAPON", value = 14 },
			{ name = INVTYPE_WEAPONMAINHAND, parseName1 = "INVTYPE_WEAPONMAINHAND", value = 18 },
			{ name = INVTYPE_WEAPONOFFHAND, parseName1 = "INVTYPE_WEAPONOFFHAND", value = 15 },
			{ name = INVTYPE_2HWEAPON, parseName1 = "INVTYPE_2HWEAPON", value = 19 },
			{ name = INVTYPE_RANGED, parseName1 = "INVTYPE_RANGED",
			  parseName2 = "INVTYPE_RANGEDRIGHT",
			  parseName3 = "INVTYPE_THROWN",
			  value = 16
			},
		},
	},
	{ name = localization.EQUIP_LOC_ACCESSORIES,
		subList = {
			-- 10 entries
			{ name = AMMOSLOT, parseName1 = "INVTYPE_AMMO", value = 22 },
			{ name = INVTYPE_CLOAK, parseName1 = "INVTYPE_CLOAK", value = 13 },
			{ name = BAGSLOT, parseName1 = "INVTYPE_BAG", value = 23 },
			{ name = INVTYPE_FINGER, parseName1 = "INVTYPE_FINGER", value = 11 },
			{ name = INVTYPE_NECK, parseName1 = "INVTYPE_NECK", value = 2 },
			{ name = INVTYPE_HOLDABLE, parseName1 = "INVTYPE_HOLDABLE", value = 21 },
			{ name = INVTYPE_BODY, parseName1 = "INVTYPE_BODY", value = 4 },
			{ name = INVTYPE_TABARD, parseName1 = "INVTYPE_TABARD", value = 17 },
			{ name = INVTYPE_TRINKET, parseName1 = "INVTYPE_TRINKET", value = 12 },
			{ name = INVTYPE_RELIC, parseName1 = "INVTYPE_RELIC", value = 24 },
		},
	},
};




LootLink.TYPES_AND_SUBTYPES = {
	-- 11 entries
	{ displayName = ARMOR, value = 1,
		subList = {
			-- 9 entries
			{ displayName = localization.DISPLAY_CLOTH,
					parseName = localization.PARSE_CLOTH,
					tooltipName = localization.TOOLTIP_CLOTH, value = 1 },
			{ displayName = localization.DISPLAY_LEATHER,
					parseName = localization.PARSE_LEATHER,
					tooltipName = localization.TOOLTIP_LEATHER, value = 2 },
			{ displayName = localization.DISPLAY_MAIL,
					parseName = localization.PARSE_MAIL,
					tooltipName = localization.TOOLTIP_MAIL, value = 4 },
			{ displayName = localization.DISPLAY_PLATE,
					parseName = localization.PARSE_PLATE,
					tooltipName = localization.TOOLTIP_PLATE, value = 5 },
			{ displayName = localization.DISPLAY_SHIELDS,
					parseName = localization.PARSE_SHIELDS,
					tooltipName = localization.TOOLTIP_SHIELDS, value = 3 },
			{ displayName = localization.DISPLAY_IDOLS,
					parseName = localization.PARSE_IDOLS,
					tooltipName = localization.TOOLTIP_IDOLS, value = 7 },
			{ displayName = localization.DISPLAY_LIBRAMS,
					parseName = localization.PARSE_LIBRAMS,
					tooltipName = localization.TOOLTIP_LIBRAMS, value = 8 },
			{ displayName = localization.DISPLAY_TOTEMS,
					parseName = localization.PARSE_TOTEMS,
					tooltipName = localization.TOOLTIP_TOTEMS, value = 9 },
			{ displayName = MISCELLANEOUS, value = 6 },
		},
	},
	
	
	{ displayName = localization.DISPLAY_WEAPONS,
		parseName = localization.PARSE_WEAPONS, value = 2,
		subList = {
			-- 16 entries
			{ displayName = localization.DISPLAY_ONEHANDED_AXES,
					parseName = localization.PARSE_ONEHANDED_AXES,
					tooltipName = localization.TOOLTIP_ONEHANDED_AXES, value = 1 },
			{ displayName = localization.DISPLAY_TWOHANDED_AXES,
					parseName = localization.PARSE_TWOHANDED_AXES,
					tooltipName = localization.TOOLTIP_TWOHANDED_AXES, value = 2 },
			{ displayName = localization.DISPLAY_ONEHANDED_SWORDS,
					parseName = localization.PARSE_ONEHANDED_SWORDS,
					tooltipName = localization.TOOLTIP_ONEHANDED_SWORDS, value = 3 },
			{ displayName = localization.DISPLAY_TWOHANDED_SWORDS,
					parseName = localization.PARSE_TWOHANDED_SWORDS,
					tooltipName = localization.TOOLTIP_TWOHANDED_SWORDS, value = 4 },
			{ displayName = localization.DISPLAY_ONEHANDED_MACES,
					parseName = localization.PARSE_ONEHANDED_MACES,
					tooltipName = localization.TOOLTIP_ONEHANDED_MACES, value = 5 },
			{ displayName = localization.DISPLAY_TWOHANDED_MACES,
					parseName = localization.PARSE_TWOHANDED_MACES,
					tooltipName = localization.TOOLTIP_TWOHANDED_MACES, value = 6 },
			{ displayName = localization.DISPLAY_POLEARMS,
					parseName = localization.PARSE_POLEARMS,
					tooltipName = localization.TOOLTIP_POLEARMS, value = 8 },
			{ displayName = localization.DISPLAY_DAGGERS,
					parseName = localization.PARSE_DAGGERS,
					tooltipName = localization.TOOLTIP_DAGGERS, value = 7 },
			{ displayName = localization.DISPLAY_FIST_WEAPONS,
					parseName = localization.PARSE_FIST_WEAPONS,
					tooltipName = localization.TOOLTIP_FIST_WEAPONS, value = 11 },
			{ displayName = localization.DISPLAY_STAVES,
					parseName = localization.PARSE_STAVES,
					tooltipName = localization.TOOLTIP_STAVES, value = 13 },
			{ displayName = localization.DISPLAY_WANDS,
					parseName = localization.PARSE_WANDS,
					tooltipName = localization.TOOLTIP_WANDS, value = 9 },
			{ displayName = localization.DISPLAY_GUNS,
					parseName = localization.PARSE_GUNS,
					tooltipName = localization.TOOLTIP_GUNS, value = 10 },
			{ displayName = localization.DISPLAY_BOWS,
					parseName = localization.PARSE_BOWS,
					tooltipName = localization.TOOLTIP_BOWS, value = 12 },
			{ displayName = localization.DISPLAY_CROSSBOWS,
					parseName = localization.PARSE_CROSSBOWS,
					tooltipName = localization.TOOLTIP_CROSSBOWS, value = 14 },
			{ displayName = localization.DISPLAY_FISHING_POLES,
					parseName = localization.PARSE_FISHING_POLES,
					tooltipName = localization.TOOLTIP_FISHING_POLES, value = 15 },
			{ displayName = MISCELLANEOUS, value = 16 },
		},
	},
	
	
	{ displayName = localization.DISPLAY_RECIPES,
		parseName = localization.PARSE_RECIPES, value = 3,
		subList = {
			-- 11 entries
			{ displayName = localization.DISPLAY_ALCHEMY,
					parseName = localization.PARSE_ALCHEMY,
					tooltipName = localization.TOOLTIP_ALCHEMY, value = 1 },
			{ displayName = localization.DISPLAY_BLACKSMITHING,
					parseName = localization.PARSE_BLACKSMITHING,
					tooltipName = localization.TOOLTIP_BLACKSMITHING, value = 4 },
			{ displayName = localization.DISPLAY_COOKING,
					parseName = localization.PARSE_COOKING,
					tooltipName = localization.TOOLTIP_COOKING, value = 5 },
			{ displayName = localization.DISPLAY_ENCHANTING,
					parseName = localization.PARSE_ENCHANTING,
					tooltipName = localization.TOOLTIP_ENCHANTING, value = 2 },
			{ displayName = localization.DISPLAY_ENGINEERING,
					parseName = localization.PARSE_ENGINEERING,
					tooltipName = localization.TOOLTIP_ENGINEERING, value = 8 },
			{ displayName = localization.DISPLAY_FIRST_AID,
					parseName = localization.PARSE_FIRST_AID,
					tooltipName = localization.TOOLTIP_FIRST_AID, value = 6 },
			{ displayName = localization.DISPLAY_FISHING,
					parseName = localization.PARSE_FISHING,
					tooltipName = localization.TOOLTIP_FISHING, value = 7 },
			{ displayName = localization.DISPLAY_JEWELCRAFTING,
					parseName = localization.PARSE_JEWELCRAFTING,
					tooltipName = localization.TOOLTIP_JEWELCRAFTING, value = 11 },
			{ displayName = localization.DISPLAY_LEATHERWORKING,
					parseName = localization.PARSE_LEATHERWORKING,
					tooltipName = localization.TOOLTIP_LEATHERWORKING, value = 9 },
			{ displayName = localization.DISPLAY_TAILORING,
					parseName = localization.PARSE_TAILORING,
					tooltipName = localization.TOOLTIP_TAILORING, value = 3 },
			{ displayName = localization.DISPLAY_BOOKS,
					parseName = localization.PARSE_BOOKS,
					tooltipName = localization.TOOLTIP_BOOKS, value = 10 },
		},
	},
	
--[[
	{ displayName = localization.DISPLAY_TRADE_GOODS,
		parseName = localization.PARSE_TRADE_GOODS, value = 5,
		subList = {
			-- 5 entries
			{ displayName = localization.DISPLAY_GEMS,
					parseName = localization.PARSE_GEMS, value = 5 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_SUB,
					parseName = localization.PARSE_TRADE_GOODS_SUB, value = 1 },
			{ displayName = localization.DISPLAY_DEVICES,
					parseName = localization.PARSE_DEVICES, value = 2 },
			{ displayName = localization.DISPLAY_EXPLOSIVES,
					parseName = localization.PARSE_EXPLOSIVES, value = 4 },
			{ displayName = localization.DISPLAY_PARTS,
					parseName = localization.PARSE_PARTS, value = 3 },
		},
	},
]]--
	-- index is 4
	{ displayName = localization.DISPLAY_TRADE_GOODS,
		parseName = localization.PARSE_TRADE_GOODS, value = 5,
		subList = {
			-- 13 entries
			{ displayName = localization.DISPLAY_TRADE_GOODS_SUB,
					parseName = localization.PARSE_TRADE_GOODS_SUB, value = 1 },
			{ displayName = localization.DISPLAY_DEVICES,
					parseName = localization.PARSE_DEVICES, value = 2 },
			{ displayName = localization.DISPLAY_EXPLOSIVES,
					parseName = localization.PARSE_EXPLOSIVES, value = 4 },
			{ displayName = localization.DISPLAY_PARTS,
					parseName = localization.PARSE_PARTS, value = 3 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_OTHER,
					parseName = localization.PARSE_TRADE_GOODS_OTHER, value = 5 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_ELEMENTAL,
					parseName = localization.PARSE_TRADE_GOODS_ELEMENTAL, value = 6 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_JEWELCRAFTING,
					parseName = localization.PARSE_TRADE_GOODS_JEWELCRAFTING, value = 7 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_LEATHER,
					parseName = localization.PARSE_TRADE_GOODS_LEATHER, value = 8 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_ENCHANTING,
					parseName = localization.PARSE_TRADE_GOODS_ENCHANTING, value = 9 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_METALSTONE,
					parseName = localization.PARSE_TRADE_GOODS_METALSTONE, value = 10 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_CLOTH,
					parseName = localization.PARSE_TRADE_GOODS_CLOTH, value = 11 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_MEAT,
					parseName = localization.PARSE_TRADE_GOODS_MEAT, value = 12 },
			{ displayName = localization.DISPLAY_TRADE_GOODS_HERB,
					parseName = localization.PARSE_TRADE_GOODS_HERB, value = 13 },
		},
	},

	{ displayName = localization.DISPLAY_REAGENTS,
		parseName = localization.PARSE_REAGENTS, value = 9 },

	--6
	{ displayName = localization.DISPLAY_CONSUMABLES,
		parseName = localization.PARSE_CONSUMABLES, value = 7,
		subList = {
			-- 9 entries
			{ displayName = localization.DISPLAY_CONSUMABLES_OTHER,
					parseName = localization.PARSE_CONSUMABLES_OTHER, value = 7 },
			{ displayName = localization.DISPLAY_CONSUMABLES_BANDAGE,
					parseName = localization.PARSE_CONSUMABLES_BANDAGE, value = 1 },
			{ displayName = localization.DISPLAY_CONSUMABLES_SCROLL,
					parseName = localization.PARSE_CONSUMABLES_SCROLL, value = 9 },
			{ displayName = localization.DISPLAY_CONSUMABLES_FLASK,
					parseName = localization.PARSE_CONSUMABLES_FLASK, value = 4 },
			{ displayName = localization.DISPLAY_CONSUMABLES_ELIXIR,
					parseName = localization.PARSE_CONSUMABLES_ELIXIR, value = 3 },
			{ displayName = localization.DISPLAY_CONSUMABLES_POTION,
					parseName = localization.PARSE_CONSUMABLES_POTION, value = 8 },
			{ displayName = localization.DISPLAY_CONSUMABLES_FOODDRINK,
					parseName = localization.PARSE_CONSUMABLES_FOODDRINK, value = 5 },
			{ displayName = localization.DISPLAY_CONSUMABLES_ITEMENHANCEMENT,
					parseName = localization.PARSE_CONSUMABLES_ITEMENHANCEMENT, value = 6 },
			{ displayName = localization.DISPLAY_CONSUMABLES,
					parseName = localization.PARSE_CONSUMABLES, value = 2 },
		},
	},

	{ displayName = localization.DISPLAY_CONTAINERS,
		parseName = localization.PARSE_CONTAINERS, value = 4,
		subList = {
			-- 10 entries
			{ displayName = localization.DISPLAY_BAGS,
					parseName = localization.PARSE_BAGS, value = 1 },
			{ displayName = localization.DISPLAY_AMMO_POUCHES,
					parseName = localization.PARSE_AMMO_POUCHES, value = 3 },
			{ displayName = localization.DISPLAY_QUIVERS,
					parseName = localization.PARSE_QUIVERS, value = 4 },
			{ displayName = localization.DISPLAY_HERB_BAGS,
					parseName = localization.PARSE_HERB_BAGS, value = 5 },
			{ displayName = localization.DISPLAY_ENCHANTING_BAGS,
					parseName = localization.PARSE_ENCHANTING_BAGS, value = 6 },
			{ displayName = localization.DISPLAY_GEM_BAGS,
					parseName = localization.PARSE_GEM_BAGS, value = 8 },
			{ displayName = localization.DISPLAY_MINING_BAGS,
					parseName = localization.PARSE_MINING_BAGS, value = 9 },
			{ displayName = localization.DISPLAY_ENGINEERING_BAGS,
					parseName = localization.PARSE_ENGINEERING_BAGS, value = 7 },
			{ displayName = localization.DISPLAY_SOUL_BAGS,
					parseName = localization.PARSE_SOUL_BAGS, value = 2 },
			{ displayName = localization.DISPLAY_LEATHERWORKING_BAGS,
					parseName = localization.PARSE_LEATHERWORKING_BAGS, value = 10 },
		},
	},


	{ displayName = localization.DISPLAY_PROJECTILES,
		parseName = localization.PARSE_PROJECTILES, value = 6,
		subList = {
			-- 3 entries
			{ displayName = localization.DISPLAY_ARROWS,
					parseName = localization.PARSE_ARROWS, value = 2 },
			{ displayName = localization.DISPLAY_BULLETS,
					parseName = localization.PARSE_BULLETS, value = 1 },
			{ displayName = localization.DISPLAY_THROWN_WEAPONS,
					parseName = localization.PARSE_THROWN_WEAPONS, value = 3 },
		},
	},


	{ displayName = localization.DISPLAY_QUEST_ITEMS,
		parseName = localization.PARSE_QUEST_ITEMS, value = 10 },


	{ displayName = localization.DISPLAY_KEYS,
		parseName = localization.PARSE_KEYS, value = 8 },

	--index is 11
	{ displayName = MISCELLANEOUS, value = 11,
		subList = {
			-- 5 entries
			{ displayName = localization.DISPLAY_MISCELLANEOUS_HOLIDAY,
					parseName = localization.PARSE_MISCELLANEOUS_HOLIDAY, value = 1 },
			{ displayName = localization.DISPLAY_MISCELLANEOUS_REAGENT,
					parseName = localization.PARSE_MISCELLANEOUS_REAGENT, value = 2 },
			{ displayName = localization.DISPLAY_MISCELLANEOUS_JUNK,
					parseName = localization.PARSE_MISCELLANEOUS_JUNK, value = 3 },
			{ displayName = localization.DISPLAY_MISCELLANEOUS_PET,
					parseName = localization.PARSE_MISCELLANEOUS_PET, value = 4 },
			{ displayName = localization.DISPLAY_MISCELLANEOUS_OTHER,
					parseName = localization.PARSE_MISCELLANEOUS_OTHER, value = 5 },
		},
	},


	{ displayName = localization.DISPLAY_TYPE_GEMS,
		parseName = localization.PARSE_TYPE_GEMS, value = 12,
		subList = {
			-- 9 entries
			{ displayName = localization.DISPLAY_RED,
					parseName = localization.PARSE_RED, value = 2 },
			{ displayName = localization.DISPLAY_BLUE,
					parseName = localization.PARSE_BLUE, value = 1 },
			{ displayName = localization.DISPLAY_YELLOW,
					parseName = localization.PARSE_YELLOW, value = 3 },
			{ displayName = localization.DISPLAY_PURPLE,
					parseName = localization.PARSE_PURPLE, value = 4 },
			{ displayName = localization.DISPLAY_ORANGE,
					parseName = localization.PARSE_ORANGE, value = 5 },
			{ displayName = localization.DISPLAY_GREEN,
					parseName = localization.PARSE_GREEN, value = 6 },
			{ displayName = localization.DISPLAY_META,
					parseName = localization.PARSE_META, value = 7 },
			{ displayName = localization.DISPLAY_SIMPLE,
					parseName = localization.PARSE_SIMPLE, value = 8 },
			{ displayName = localization.DISPLAY_PRISMATIC,
					parseName = localization.PARSE_PRISMATIC, value = 9 },
		},
	},
};




LootLink.BASEITEM_METATABLE = {
	__index = (
	  function(self, key)
		return LootLink:Compatibility_GetBaseTableIndex(self, key);
	  end
	);
	
	
	__newindex = (
	  function(self, key, value)
		LootLink:Compatibility_SetBaseTableIndex(self, key, value);
	  end
	);
};




LootLink.SUFFIXITEM_METATABLE = {
	__index = (
	  function(self, key)
		return LootLink:Compatibility_GetSuffixTableIndex(self, key);
	  end
	);
	
	
	__newindex = (
	  function(self, key, value)
		LootLink:Compatibility_SetSuffixTableIndex(self, key, value);
	  end
	);
};




LootLink.DATABASE_METATABLE = {
	__index = (
	  function(ItemLinks, key)
		return LootLink:Compatibility_GetDatabaseIndex(key);
	  end
	);
};



LootLink.LOOTLINK_STATE_METATABLE = {
	__index = (
	  function(LootLinkState, key)
		return LootLink:Compatibility_GetLootLinkStateIndex(key);
	  end
	);
};




do
	--local startTime, startMem = GetTime(), collectgarbage("count");
	
	
	local getItemInfo = GetItemInfo;
	local baseName;
	local baseID = 25; -- "Worn Shortsword", a very common item with the lowest known base ID.
	
	while (baseID < 34000 and baseName == nil) do
		baseName = getItemInfo(baseID);
		baseID = (baseID + 1);
	end
	
	baseID = (baseID - 1);
	
	
	
	local suffixIDsToNameKeys = {};
	local suffixNamesToNameKeys = {};
	local suffixNameKeysToNames = {};
	
	--PrintComma(baseID, baseName);
	
	
	if (baseName ~= nil) then
		local stringgsub = LootLink.STRING_LIBRARY.gsub;
		local stringtrim = LootLink.STRING_LIBRARY.trim;
		local tableinsert = LootLink.TABLE_LIBRARY.insert;
		
		local linkPrefix = ("item:"..baseID..":0:0:0:0:0:");
		
		local increment = 1;
		
		local nameWithSuffix;
		local suffix;
		
		
		for index = -70, 2200 do
			nameWithSuffix = getItemInfo(linkPrefix..index..":0");
			
			if (nameWithSuffix ~= baseName) then
				suffix = stringtrim((stringgsub(nameWithSuffix, baseName, "", 1)));
				
				if (suffix ~= "") then
					if (suffixNamesToNameKeys[suffix] == nil) then
						suffixNamesToNameKeys[suffix] = increment;
						tableinsert(suffixNameKeysToNames, suffix);
						increment = (increment + 1);
					end
					
					suffixIDsToNameKeys[index] = suffixNamesToNameKeys[suffix];
				end
			end
		end
		
		
	else
		StaticPopupDialogs["LOOTLINK_NO_CACHED_ITEMS"] = {
			text = localization.ERROR_NO_CACHED_ITEMS_POPUP,
			button1 = localization.ERROR_NO_CACHED_ITEMS_RELOAD,
			button2 = IGNORE,
			OnAccept = (function() ReloadUI(); end),
			timeout = 0,
			whileDead = 1,
		};
		
		StaticPopup_Show("LOOTLINK_NO_CACHED_ITEMS");
		
		DEFAULT_CHAT_FRAME:AddMessage(localization.ERROR_NO_CACHED_ITEMS, 0.0, 0.8, 0.8);
	end
	
	
	
	LootLink.SUFFIX_IDS_TO_NAME_KEYS = suffixIDsToNameKeys;
	LootLink.SUFFIX_NAMES_TO_NAME_KEYS = suffixNamesToNameKeys;
	LootLink.SUFFIX_NAME_KEYS_TO_NAMES = suffixNameKeysToNames;
	
	
	--PrintComma(GetTime() - startTime, collectgarbage("count") - startMem);
end




localization = nil;

