

local LootLink = LootLink;




LootLink.DatabaseSize = 0;


LootLink.RecentlyAddedItems = {};



StaticPopupDialogs["LOOTLINK_CONFIRM_REMOVEITEM"] = {
	text = LootLink.LOCALIZATION.REMOVE_ITEM_CONFIRM,
	button1 = YES,
	button2 = NO,
	OnAccept = (function()
		local LootLink = LootLink;
		if (LootLink.ItemToRemove.suffixID) then
			LootLink:ItemTables_RemoveSuffixItem(LootLink.ItemToRemove.itemTable, LootLink.ItemToRemove.suffixID, LootLink.ItemToRemove.instanceID, true);
		else
			LootLink:ItemTables_RemoveBaseItem(LootLink.ItemToRemove.itemTable[-1], true);
		end
		LootLink.ItemToRemove = nil;
		LootLink:Results_FullRebuild();
	end),
	timeout = 0,
	exclusive = 0,
	whileDead = 1,
	showAlert = 1
};




function LootLink:ItemTables_Populate(baseID, suffixID, instanceID)
	local itemTable;
	
	if (type(baseID) == "table") then
		itemTable = baseID;
		baseID = itemTable[-1];
	end
	
	instanceID = bit.band((instanceID or 0), 65535)  --Siz

	-- Start with basic information available in the cache.
	local itemName, _, qualityNumber, realLevel, minLevel, itemType, itemSubtype, _, equipLoc, texture = GetItemInfo(
	  "item:"..baseID..":0:0:0:0:0:"..(suffixID or 0)..":"..(instanceID or 0)
	);
	
	-- Don't proceed if this item isn't actually in the cache.
	if (not itemName) then return nil; end
	
	
	if (not itemTable) then itemTable = self.Database[baseID]; end
	
	
	if (self.Config.StoreItemTexturePath) then
		itemTable[13] = texture;
	end
	
	
	local suffix;
	
	if (suffixID and suffixID ~= 0) then
		suffix = self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]];
		
		if (suffix) then
			itemName = (itemName:gsub(suffix, "", 1)):trim();
		--else
			--PrintComma("unrecognized suffix in Populate", baseID, itemName, suffixID, self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]);
		end
	else
		itemTable[1] = itemName;
	end
	
	
	itemTable[2] = qualityNumber;
	
	
	itemTable[10] = minLevel;
	itemTable[12] = realLevel;
	
	
	itemTable[8] = nil;
	itemTable[9] = nil;
	
	local typeTable, subtypeTable = self:ItemTables_GetTypeAndSubtypeTables(itemType, itemSubtype);
	
	if (typeTable) then
		itemTable[8] = typeTable.value;
		
		if (subtypeTable) then
			itemTable[9] = subtypeTable.value;
			
		elseif (subtypeTable == false) then
			DEFAULT_CHAT_FRAME:AddMessage(
			  self.LOCALIZATION.REPORT_UNRECOGNIZED_SUBTYPE:format(itemType, (itemSubtype or self.LOCALIZATION.GENERIC_NONE), baseID),
        0.0, 0.8, 0.8
			);
		end
		
	else
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.LOCALIZATION.REPORT_UNRECOGNIZED_TYPE:format((itemType or self.LOCALIZATION.GENERIC_NONE), baseID),
		  0.0, 0.8, 0.8
		);
	end
	
	
	itemTable[7] = nil;
	
	local equipTable = self:ItemTables_GetEquipLocationTable(equipLoc);
	
	if (equipTable) then
		itemTable[7] = equipTable.value;
		
	elseif (equipTable == false) then
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.LOCALIZATION.REPORT_UNRECOGNIZED_EQUIPLOC:format((equipLoc or self.LOCALIZATION.GENERIC_NONE), itemName),
		  0.0, 0.8, 0.8
		);
	end
	
	
	if (not itemTable[3] or (suffixID and (suffixID > 0 or instanceID))) then
		self:ItemTables_ParseTooltip(itemTable, suffixID, instanceID);
	end
	
	
	return itemTable;
end




function LootLink:ItemTables_ParseTooltip(itemTable, suffixID, instanceID)
	local tooltip = self.PARSING_TOOLTIP;
	
	tooltip:ClearLines();
	tooltip:SetHyperlink("item:"..itemTable[-1]..":0:0:0:0:0:0:0");
	
	itemTable[4], itemTable[11] = self:ItemTables_GetBindTypeAndUnique(tooltip);
	
	
	if (self.Config.LightMode) then return; end -- The rest of the function deals with parsing full-text.
	
	
	--[[
	It is important to note that the index within which each line of text is stored is only relevant to the indexes of the
	immediately preceding and following lines of text. It is used only to track which lines come before and after the
	current line. Therefore, the tooltip must be built in sequential order to be accurate; the line data cannot be accessed
	in an arbitrary order with the expectation of receiving a certain piece of text.
	
	The line indexes should, however, be stored contiguously from the start of the tooltip up to the point at which the
	random suffix lines enter in. There is a deliberately nil index thereafter, which informs the rebuilding code to look
	for the suffix portion of the full text. After that the lines should again be contiguous until the end of the tooltip.
	
	TextRight strings are stored in whatever negative index complements the current TextLeft string's positive index.
	]]--
	
	
	local suffixText;
	
	if (suffixID and suffixID ~= 0 and itemTable[3]) then
		if (suffixID < 0) then suffixText = itemTable[3][suffixID][instanceID]; else suffixText = itemTable[3][suffixID]; end
		
		if (type(suffixText) == "table") then
			local oldValue = suffixText[0];
			
			for index, text in pairs(suffixText) do
				suffixText[index] = nil;
			end
			
			suffixText[0] = oldValue;
			
		else
			suffixText = { [0] = suffixText; }; -- Copy over the previous display setting, true or false, from the old value.
			if (suffixID < 0) then itemTable[3][suffixID][instanceID] = suffixText; else itemTable[3][suffixID] = suffixText; end
		end
	end
	
	
	local fullText = itemTable[5];
	
	if (fullText) then
		for index, text in pairs(fullText) do
			fullText[index] = nil;
		end
	else
		fullText = {};
		itemTable[5] = fullText;
	end
	
	
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local globalEnv = self.GLOBAL_ENVIRONMENT;
	local tooltipPointers = tooltip.LINE_POINTERS;
	
	local spellKnown = ITEM_SPELL_KNOWN;
	
	local textMemory = (suffixID and self:Main_GetReusableTable(1));
	
	local textLeft, textRight;
	
	
	
	for index = 2, tooltip:NumLines() do
		textLeft = tooltipPointers[index];
		
		if (not textLeft) then
			textLeft = globalEnv["LootLink_ParsingTooltipTextLeft"..index];
			tooltipPointers[index] = textLeft;
		end
		
		textLeft = textLeft:GetText();
		
		
		if (textLeft ~= spellKnown) then
			-- TextLeft string are stored in positive sequential order.
			if (suffixID and suffixID ~= 0) then
				textMemory[textLeft] = true;
			else
				tableinsert(fullText, textLeft);
			end
		end
		
		
		textRight = tooltipPointers[-index];
		
		if (not textRight) then
			textRight = globalEnv["LootLink_ParsingTooltipTextRight"..index];
			tooltipPointers[-index] = textRight;
		end
		
		if (textRight:IsShown()) then
			-- This is not stored in textMemory because the suffix portion of an item never has any TextRight strings.
			fullText[-index] = textRight:GetText();
		end
	end
	
	
	if (suffixID and suffixID ~= 0) then
		tooltip:ClearLines();
		tooltip:SetHyperlink("item:"..itemTable[-1]..":0:0:0:0:0:"..suffixID..":"..(instanceID or 0));
		
		local flagInserted = nil;
		
		
		for index = 2, tooltip:NumLines() do
			textLeft = tooltipPointers[index];
			
			if (not textLeft) then
				textLeft = globalEnv["LootLink_ParsingTooltipTextLeft"..index];
				tooltipPointers[index] = textLeft;
			end
			
			textLeft = textLeft:GetText();
			
			
			if (textMemory[textLeft]) then
				tableinsert(fullText, textLeft);
				
			else
				if (not flagInserted) then
					tableinsert(fullText, false);
					flagInserted = true;
				end
				
				tableinsert(suffixText, textLeft);
			end
			
		end
	end
	
	
	if (next(fullText) == nil) then
		itemTable[5] = nil;
	end
	
	if (suffixID ~= nil and suffixID ~= 0 and (next(suffixText) == nil or (suffixText[0] and next(suffixText, 0) == nil))) then
		if (suffixID > 0) then
			itemTable[3][suffixID] = (suffixText[0] or false);
		else
			itemTable[3][suffixID][instanceID] = (suffixText[0] or false);
		end
	end
end



function LootLink:ItemTables_GetConcatenatedFullText(itemTable, suffixID, instanceID, suffixName)
	if (not itemTable[5]) then return (""); end
	
	
	local stringjoin = self.STRING_LIBRARY.join;
	local ipairs = ipairs;
	
	local fullTextTable = itemTable[5];
	
	local concatString;
	if (not suffixID) then
		concatString = itemTable[1];
	else
		concatString = stringjoin(" ", itemTable[1], self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]]);
	end
	
	local currentLine;
	
	
	for index = 2, self.TABLE_LIBRARY.maxn(fullTextTable) do
		currentLine = fullTextTable[index];
		
		if (currentLine == false) then
			if (suffixID) then
				local suffixTextTable = (type(itemTable[3][suffixID]) == "table" and ((suffixID > 0 and itemTable[3][suffixID]) or itemTable[3][suffixID][instanceID]));
				
				if (type(suffixTextTable) == "table") then
					for subIndex, lineText in ipairs(suffixTextTable) do
						concatString = stringjoin("\\n", concatString, lineText);
					end
				end
			end
			
		else
			concatString = stringjoin("\\n", concatString, (fullTextTable[index] or ""), (fullTextTable[-index] or ""));
		end
	end
	
	
	return concatString;
end




function LootLink:ItemTables_GetTypeAndSubtypeTables(itemType, itemSubtype)
	if (itemType == (self.LOCALIZATION.PARSE_WEAPONS or self.LOCALIZATION.DISPLAY_WEAPONS)
	  and itemSubtype == (self.LOCALIZATION.PARSE_THROWN_WEAPONS or self.LOCALIZATION.DISPLAY_THROWN_WEAPONS)
	) then
		-- Special case for thrown weapons (which get sorted with projectiles).
		return self.TYPES_AND_SUBTYPES[8], self.TYPES_AND_SUBTYPES[8].subList[3];
		
		
	elseif (itemType == (self.LOCALIZATION.PARSE_QUIVERS or self.LOCALIZATION.DISPLAY_QUIVERS)) then
		-- Special case for quivers (which get sorted with containers).
		return self.TYPES_AND_SUBTYPES[7], self.TYPES_AND_SUBTYPES[7].subList[3];
		
		
	else
		local pairs = pairs;
		
		for typeIndex, typeTable in pairs(self.TYPES_AND_SUBTYPES) do
			if ((typeTable.parseName or typeTable.displayName) == itemType) then
				
				if (typeTable.subList) then
					for subtypeIndex, subtypeTable in pairs(typeTable.subList) do
						if ((subtypeTable.parseName or subtypeTable.displayName) == itemSubtype) then
							return typeTable, subtypeTable;
						end
					end
					
					-- Type table had a sublist, meaning there should've been a match in the inner loop. There wasn't, so indicate that this subitem is not recognized.
					return typeTable, false;
				end
				
				return typeTable, nil;
			end
		end
	end
	
	
	return nil, nil;
end




function LootLink:ItemTables_GetEquipLocationTable(equipLoc)
	if (not equipLoc or equipLoc == "") then return nil; end
	
	
	local pairs = pairs;
	
	for categoryIndex, categoryTable in pairs(self.EQUIP_LOCATIONS) do
		for equipIndex, equipTable in pairs(categoryTable.subList) do
			
			if (
			  equipTable.parseName1 == equipLoc
			  or equipTable.parseName2 == equipLoc
			  or equipTable.parseName3 == equipLoc
			) then
				return equipTable;
			end
			
		end
	end
	
	
	-- False indicates that there was no match but equipLoc was a valid string.
	return false;
end




function LootLink:ItemTables_GetBindTypeAndUnique(tooltip)
	local tooltipName = tooltip:GetName();
	local globalEnv = self.GLOBAL_ENVIRONMENT;
	
	
	local textLeft = globalEnv[tooltipName.."TextLeft3"]:GetText();
	
	if (textLeft
	  and (textLeft == ITEM_UNIQUE
	   or textLeft == "Unique-Equipped"
	   or textLeft:match(self.LOCALIZATION.PARSE_UNIQUE_MULTIPLE)
	  )
	) then
		textLeft = globalEnv[tooltipName.."TextLeft2"]:GetText();
		
		for bindIndex, bindTable in pairs(self.BIND_DATA) do
			if (globalEnv[bindTable.parseName] == textLeft) then
				return bindIndex, true;
			end
		end
		
		return nil, true;
	end
	
	
	textLeft = globalEnv[tooltipName.."TextLeft2"]:GetText();
	
	if (textLeft
	  and (textLeft == ITEM_UNIQUE
	   or textLeft == "Unique-Equipped"
	   or textLeft:match(self.LOCALIZATION.PARSE_UNIQUE_MULTIPLE)
	  )
	) then
		-- If this is the case, there is no binding text, because the Unique line always appears below the Bind Type line.
		return nil, true;
	end
	
	
	for bindIndex, bindTable in pairs(self.BIND_DATA) do
		if (globalEnv[bindTable.parseName] == textLeft) then
			return bindIndex, nil;
		end
	end
	
	return nil, nil;
end




function LootLink:ItemTables_AddBaseItem(baseID, itemName, rarityNumber, feedback)
	assert(type(itemName) == "string", "itemName is not a string");
	
	
	rarityNumber = tonumber(rarityNumber);
	
	if (rarityNumber == nil or rarityNumber < (self.Config.MinStoredRarity or 0) or rarityNumber > 7) then
		if (feedback) then
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ADD_ITEM_INVALID_RARITY, 0.0, 0.8, 0.8);
		end
		
		return nil;
	end
	
	
	local itemTable = self.Database[baseID];
	
	
	if (itemTable) then
		if (feedback) then
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ADD_ITEM_ALREADY_PRESENT:format(baseID, itemTable[1]), 0.0, 0.8, 0.8);
		end
		
		return itemTable;
	end
	
	
	local realName, _, realRarity = GetItemInfo(baseID);
	
	if (realName) then
		itemName = realName;
		rarityNumber = realRarity;
	end
	
	itemTable = {
		[-1] = baseID;
		[1] = itemName;
		[2] = rarityNumber;
	};
	
	self.Database[baseID] = itemTable;
	self:ItemTables_Populate(itemTable, 0);
	setmetatable(itemTable, self.BASEITEM_METATABLE);
	
	self.TABLE_LIBRARY.insert(self.RecentlyAddedItems, 1, (ITEM_QUALITY_COLORS[(itemTable[2] < 7 and itemTable[2]) or 1].hex..itemTable[1].."|r"));
	
	if (#self.RecentlyAddedItems == (self.MAX_RECENTLY_ADDED_ITEMS + 1)) then
		self.TABLE_LIBRARY.remove(self.RecentlyAddedItems);
	end
	
	self.DatabaseSize = (self.DatabaseSize + 1);
	self:Results_UpdateTitle(true);
	
	
	if (feedback) then
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.LOCALIZATION.ADD_ITEM_SUCCESS:format(baseID, self.GLOBAL_ENVIRONMENT["ITEM_QUALITY"..rarityNumber.."_DESC"], itemName),
		  0.0, 0.8, 0.8
		);
	end
	
	
	return itemTable;
end



function LootLink:ItemTables_AddBaseItems_FromGetItemInfoLoop(startID, endID)
	local getItemInfo = GetItemInfo;
	local addBase = self.ItemTables_AddBaseItem;
	
	local database = self.Database;
	
	local itemName, _, qualityNumber;
	
	
	for baseID = startID, endID do
		
		if (not database[baseID]) then
			itemName, _, qualityNumber = getItemInfo(baseID);
			
			if (itemName) then
				addBase(self, baseID, itemName, qualityNumber);
			end
		end
		
	end
end



local function LootLink_ItemTables_AddBaseItem_ByRequestLink_OnUpdate(self, elapsed)
	self.Elapsed = (self.Elapsed + elapsed);
	
	if (self.Elapsed < 2.0) then return; end
	
	self.Elapsed = 0.0;
	
	
	LootLink_ParsingTooltip:ClearLines();
	LootLink_ParsingTooltip:SetHyperlink("item:"..self.baseID..":0:0:0:0:0:0:0");
	
	if (self.firstPass) then
		self.firstPass = nil;
		return;
	end
	
	
	local name, _, rarity = GetItemInfo(self.baseID);
	
	local itemTable = LootLink:ItemTables_AddBaseItem(self.baseID, name, rarity, true);
	
	
	self.hyperlink = nil;
	self.feedback = nil;
	
	if (self.finishFunc) then self.finishFunc(self, itemTable); end
	self.finishFunc = nil;
	
	
	self:Hide();
end



function LootLink:ItemTables_AddBaseItem_ByRequestLink(baseID, feedback, finishFunc)
	baseID = tonumber(baseID);
	
	local itemTable = self.Database[baseID];
	
	if (itemTable) then
		if (feedback) then
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ADD_ITEM_ALREADY_PRESENT:format(baseID, itemTable[1]), 0.0, 0.8, 0.8);
		end
		
		return itemTable;
	end
	
	
	-- If we get to this point, it is fairly safe to assume that the item is not in the player's local cache, because if it was, it would already be in the database from the login GetItemInfo scan.
	
	local frame = LootLink_AddByRequestLink_OnUpdateFrame;
	
	if (not frame) then
		frame = CreateFrame("Frame", "LootLink_AddByRequestLink_OnUpdateFrame", WorldFrame);
		frame:SetScript("OnUpdate", LootLink_ItemTables_AddBaseItem_ByRequestLink_OnUpdate);
		frame.Elapsed = 0.0;
		
	elseif (frame:IsShown()) then
		return false; -- False indicates that an operation is already in progress and you must wait.
	end
	
	
	frame.baseID = baseID;
	frame.feedback = feedback;
	frame.finishFunc = finishFunc;
	frame.firstPass = true;
	
	frame:Show();
	
	
	if (feedback) then
		DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ADD_ITEM_BY_REQUEST_LINK_STARTING:format(baseID), 0.0, 0.8, 0.8);
	end
	
	
	return true; -- True indicates that the frame has been shown and the hyperlink will be set in exactly two seconds.
end



function LootLink:ItemTables_RemoveBaseItem(baseID, feedback)
	local itemTable = self.Database[baseID];
	
	
	if (feedback) then
		DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.REMOVE_ITEM_DONE:format(itemTable[1]), 0.0, 0.8, 0.8);
	end
	
	
	if (itemTable[3]) then
		local pairs = pairs;
		
		local count = 1;
		
		for suffixID, suffixData in pairs(itemTable[3]) do
			if (suffixID < 0) then
				for instanceID, instanceData in pairs(suffixData) do
					count = (count + 1);
				end
			else
				count = (count + 1);
			end
		end
		
		self.DatabaseSize = (self.DatabaseSize - count);
		
	else
		self.DatabaseSize = (self.DatabaseSize - 1);
	end
	
	
	self.Database[baseID] = nil;
end




function LootLink:ItemTables_AddSuffixItem(itemTable, suffixID, instanceID)
	suffixID = tonumber(suffixID);
	instanceID = tonumber(instanceID);
  instanceID = bit.band(instanceID, 65535)	
	
	if (not itemTable[3]) then
		itemTable[3] = {};
	end
	
	
	local isNew = nil;
	
	if (suffixID > 0) then
		if (itemTable[3][suffixID] == nil) then
			itemTable[3][suffixID] = false;
			isNew = true;
		end
		
	else
		if (itemTable[3][suffixID] == nil) then
			itemTable[3][suffixID] = { [instanceID] = false; };
			isNew = true;
			
		elseif (itemTable[3][suffixID][instanceID] == nil) then
			itemTable[3][suffixID][instanceID] = false;
			isNew = true;
		end
	end
	
	
	if (isNew) then
		self:ItemTables_Populate(itemTable, suffixID, instanceID);
		
		self.TABLE_LIBRARY.insert(
		  self.RecentlyAddedItems,
		  1,
		  (
		    ITEM_QUALITY_COLORS[(itemTable[2] < 7 and itemTable[2]) or 1].hex
			..itemTable[1]
			.." "
			..self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]]
			.."|r"
		  )
		);
		
		if (#self.RecentlyAddedItems == (self.MAX_RECENTLY_ADDED_ITEMS + 1)) then
			self.TABLE_LIBRARY.remove(self.RecentlyAddedItems);
		end
		
		self.DatabaseSize = (self.DatabaseSize + 1);
		self:Results_UpdateTitle(true);
	end
	
	
	if (suffixID > 0) then
		return itemTable[3][suffixID];
	else
		return itemTable[3][suffixID][instanceID];
	end
end



function LootLink:ItemTables_RemoveSuffixItem(itemTable, suffixID, instanceID, feedback)
	if (not itemTable or not itemTable[3] or not itemTable[3][suffixID]) then return; end
	
	
	if (feedback) then
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.LOCALIZATION.REMOVE_ITEM_DONE:format(itemTable[1].." "..self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]]),
		  0.0, 0.8, 0.8
		);
	end
	
	
	if (instanceID and instanceID ~= 0) then
		if (itemTable[3][suffixID][instanceID] ~= nil) then
			itemTable[3][suffixID][instanceID] = nil;
			self.DatabaseSize = (self.DatabaseSize - 1);
		end
		
	else
		itemTable[3][suffixID] = nil;
		self.DatabaseSize = (self.DatabaseSize - 1);
	end
	
	
	if (suffixID > 0) then
		if (next(itemTable[3])) then
			return true;
		else
			return false;
		end
		
	else
		if (itemTable[3][suffixID] and next(itemTable[3][suffixID])) then
			return true;
		else
			return false;
		end
	end
end




local function LootLink_GetStringByteForWholeString(inputString)
	local stringbyte = LootLink.STRING_LIBRARY.byte;
	
	local total = 0;
	
	
	for index = 1, inputString:len() do
		total = (total + stringbyte(inputString, index, index));
	end
	
	
	return total;
end



function LootLink:ItemTables_GetBaseTableHash(itemTable)
	local getStringByteForWholeString = LootLink_GetStringByteForWholeString;
	
	local hash = (
	  getStringByteForWholeString(itemTable[1])
	  + itemTable[2]
	  + ((itemTable[3] and 1) or 0)
	  + (itemTable[4] or 0)
	  + (itemTable[6] or 0)
	  + (itemTable[7] or 0)
	  + (itemTable[8] or 0)
	  + (itemTable[9] or 0)
	  + (itemTable[10] or 0)
	  + ((itemTable[11] == true and 1) or 0)
	  + (itemTable[12] or 0)
	);
	
	
	if (itemTable[5]) then
		for index, text in pairs(itemTable[5]) do
			if (text == false) then
				hash = (hash + index);
			else
				hash = (hash + getStringByteForWholeString(text));
			end
		end
	end
	
	
	return hash;
end

