

local LootLink = LootLink;




do
	local frame = LootLink.SCRIPTS_FRAME;
	local register = frame.RegisterEvent;
	
	register(frame, "VARIABLES_LOADED");
	register(frame, "PLAYER_ENTERING_WORLD");
	register(frame, "PLAYER_LOGOUT");
	
	register(frame, "CHAT_MSG_SYSTEM");
	register(frame, "CHAT_MSG_SAY");
	register(frame, "CHAT_MSG_YELL");
	register(frame, "CHAT_MSG_LOOT");
	register(frame, "CHAT_MSG_CHANNEL");
	register(frame, "CHAT_MSG_GUILD");
	register(frame, "CHAT_MSG_OFFICER");
	register(frame, "CHAT_MSG_RAID");
	register(frame, "CHAT_MSG_PARTY");
	register(frame, "CHAT_MSG_WHISPER");
	register(frame, "CHAT_MSG_WHISPER_INFORM");
	
	register(frame, "BANKFRAME_OPENED");
	register(frame, "AUCTION_HOUSE_SHOW");
	register(frame, "AUCTION_HOUSE_CLOSED");
	register(frame, "AUCTION_ITEM_LIST_UPDATE");
	register(frame, "TRADE_SHOW");
	register(frame, "TRADE_TARGET_ITEM_CHANGED");
	register(frame, "TRADE_PLAYER_ITEM_CHANGED");
	register(frame, "MERCHANT_SHOW");
	register(frame, "UPDATE_MOUSEOVER_UNIT");
	register(frame, "PLAYER_TARGET_CHANGED");
	register(frame, "UNIT_INVENTORY_CHANGED");
	register(frame, "MAIL_INBOX_UPDATE");
end




function LootLink:Main_OnEvent(event, ...)
	if (event == "UPDATE_MOUSEOVER_UNIT") then
		self:Parsing_ParseInventory("mouseover");
		
		
	elseif (event == "PLAYER_TARGET_CHANGED") then
		self:Parsing_ParseInventory("target");
		
		
	elseif (event == "CHAT_MSG_ADDON") then
		local prefix, message, distType, sender = select(1, ...);
		
		if (prefix == self.DATASHARING_CHANNEL_NAME) then
			self:DataSharing_ParseMessage(message, sender);
		end
		
		
	elseif (event:sub(1, 4) == "CHAT") then
		local message, sender, _, _, _, _, _, _, channelName = select(1, ...);
		
		self:Parsing_ParseFromString(message);
		
		if (event == "CHAT_MSG_CHANNEL") then
			if (self.Config.EnableDataSharing and channelName:lower() == self.DATASHARING_CHANNEL_NAME) then
				self:DataSharing_ParseMessage(message, sender);
			end
			
		elseif (self.Config.EnableAutoLinkBot) then
			self:AutoLinkBot_ParseRequest(message, sender, event);
		end
		
		
	elseif (event == "AUCTION_ITEM_LIST_UPDATE") then
		self:Parsing_ParseCurrentAuctionList();
		
		
	elseif (event == "AUCTION_HOUSE_SHOW") then
		if (self.DoAuctionScan) then
			self:AuctionScanning_StartScan();
		end
		
		
	elseif (event == "AUCTION_HOUSE_CLOSED") then
		if (self.DoAuctionScan) then
			self:AuctionScanning_OnAuctionHouseClosed();
		end
		
		
	elseif (event == "MERCHANT_UPDATE" or event == "MERCHANT_SHOW") then
		self:Parsing_ParseMerchant();
		self:Parsing_ParseSellPrices();
		
		
	elseif (event == "BANKFRAME_OPENED") then
		self:Parsing_ParseBank();
		
		
	elseif (event == "TRADE_SHOW" or event == "TRADE_TARGET_ITEM_CHANGED" or event == "TRADE_PLAYER_ITEM_CHANGED") then
		self:Parsing_ParseTradeWindow();
		
		
	elseif (event == "UNIT_INVENTORY_CHANGED") then
		local unit = select(1, ...);
		
		if (unit == "player") then
			self:Parsing_ParseBags();
		end
		
		self:Parsing_ParseInventory(unit);


	elseif (event == "MAIL_INBOX_UPDATE") then
		self:Parsing_ParseInbox();


	elseif (event == "PLAYER_ENTERING_WORLD") then
		self:Parsing_ParseBags();
		self:Parsing_ParseInventory("player");

		local delays = self.UnitInventoryMouseoverParseDelays;

		for name in pairs(delays) do
			delays[name] = nil;
		end


	elseif (event == "VARIABLES_LOADED") then
		self:Main_InitDatabase();
		self:Main_InitConfig();
		self:Main_ApplyConfig();
		self:ItemTables_AddBaseItems_FromGetItemInfoLoop(1, 36000);
		
		
	elseif (event == "PLAYER_LOGOUT") then
		local pairs = pairs;
		local type = type;
		
		-- Clean up the database's runtime keys.
		for baseID, itemTable in pairs(self.Database) do
			itemTable[-1] = nil; -- Copy of base ID.
			itemTable[-2] = nil; -- Flag for expanded/unexpanded in results list.
			itemTable[-3] = nil; -- Number of suffixes that will be shown/hidden.
			
			if (itemTable[3] and itemTable[3] ~= {}) then
				for suffixID, suffixData in pairs(itemTable[3]) do
					if (suffixID < 0) then
						for instanceID, instanceData in pairs(suffixData) do
							if (type(instanceData) == "table") then
								instanceData[0] = nil;
							elseif (instanceData ~= false) then
								suffixData[instanceID] = false;
							end
						end
					else
						if (type(suffixData) == "table") then
							suffixData[0] = nil;
						elseif (suffixData ~= false) then
							itemTable[3][suffixID] = false;
						end
					end
				end
			end
		end
		
		-- Remove the compatibility placeholder.
		ItemLinks = nil;
		
		-- Make sure the results frame's position isn't stored if it's not supposed to be.
		if (not self.Config.MobilizeResults) then
			self.RESULTS_FRAME:SetUserPlaced(false);
		end
		
		-- Save the ToggleButton's position.
		if (self.TOGGLE_BUTTON) then
			self.Config.ToggleButtonX, self.Config.ToggleButtonY = self.TOGGLE_BUTTON:GetCenter();
		end
		
		-- Leave the DataSharing channel.
		if (self.DATASHARING_CHANNEL_NAME) then
			LeaveChannelByName(self.DATASHARING_CHANNEL_NAME);
		end
	end
end


LootLink.SCRIPTS_FRAME:SetScript("OnEvent", (function(self, event, ...) self.LOOTLINK:Main_OnEvent(event, ...); end));




function LootLink.ReplaceHook_ChatFrame_MessageEventHandler(event)
	if (event == "CHAT_MSG_CHANNEL") then
		if (arg9 == LootLink.DATASHARING_CHANNEL_NAME) then
			return true;
		end
		
	elseif (event == "CHAT_MSG_WHISPER_INFORM") then
		if (arg1:sub(1, 2) == "!-") then
			return true;
		end
	end
	
	return LootLink.Original_ChatFrame_MessageEventHandler(event);
end




function LootLink:Main_SlashCommandHandler(msg)
	if (msg == "") then
		self:Results_Toggle();
		return;
	end
	
	
	local command, argument = msg:match("([^%s]+) (.+)");
	command = (command or msg):lower();
	
	local localization = self.LOCALIZATION;
	
	
	if (command == localization.SLASH_SEARCH or command == localization.SLASH_SEARCH_SHORT) then
		if (argument) then
			local pairs = pairs;
			local stringconcat = self.STRING_LIBRARY.concat;
			local getItemInfo = GetItemInfo;
			local getHyperlinkByItemTableAndSuffixID = self.Matching_GetHyperlinkByItemTableAndSuffixID;
			
			local items = self:Matching_ByCharsInName(self:Main_GetReusableTable(1), true, self.MAX_COMMAND_LINE_SEARCH_MATCHES, argument, nil);
			
			local message = self.LOCALIZATION.CHAT_SEARCH_RESULTS:format(argument);
			local cached;
			
			
			for baseID, itemTable in pairs(items) do
				
				cached = getItemInfo(baseID);
				
				if (cached) then
					message = stringconcat(message, " ", getHyperlinkByItemTableAndSuffixID(self, itemTable));
				else
					message = stringconcat(message, " *", getHyperlinkByItemTableAndSuffixID(self, itemTable), "*");
				end
				
				
				if (itemTable[3]) then
					if (cached) then
						for suffixID, suffixData in pairs(itemTable[3]) do
							if (suffixID < 0) then
								for instanceID, instanceData in pairs(suffixData) do
									message = stringconcat(message, " ", getHyperlinkByItemTableAndSuffixID(self, itemTable, suffixID, instanceID));
								end
							else
								message = stringconcat(message, " ", getHyperlinkByItemTableAndSuffixID(self, itemTable, suffixID));
							end
						end
					else
						for suffixID, suffixData in pairs(itemTable[3]) do
							if (suffixID < 0) then
								for instanceID, instanceData in pairs(suffixData) do
									message = stringconcat(message, " *", getHyperlinkByItemTableAndSuffixID(self, itemTable, suffixID, instanceID), "*");
								end
							else
								message = stringconcat(message, " *", getHyperlinkByItemTableAndSuffixID(self, itemTable, suffixID), "*");
							end
						end
					end
				end
				
			end
			
			
			DEFAULT_CHAT_FRAME:AddMessage(message, 0.0, 0.8, 0.8);
		end
		
		
	elseif (command == localization.SLASH_ADDBYBASICDETAILS) then
		local baseID, rarityNumber, itemName = argument:match("(%d+) (%d) (.+)");
		
		if (baseID) then
			self:ItemTables_AddBaseItem(baseID, itemName, rarityNumber, true);
		else
			DEFAULT_CHAT_FRAME:AddMessage(localization.ADD_ITEM_BY_BASIC_DETAILS_SYNTAX_HELP, 0.0, 0.8, 0.8);
		end
		
		
	elseif (command == localization.SLASH_ADDBYREQUESTLINK) then
		local baseID = tonumber(argument);
		
		if (baseID) then
			self:ItemTables_AddBaseItem_ByRequestLink(baseID, true);
		else
			DEFAULT_CHAT_FRAME:AddMessage(localization.ADD_ITEM_BY_REQUEST_LINK_SYNTAX_HELP, 0.0, 0.8, 0.8);
		end
		
		
	elseif (command == localization.SLASH_AUCTIONSCAN) then
		if (self:Main_LoadPlugin("AuctionScanning", "AuctionScanning_StartScan")) then
			self.DoAuctionScan = true;
			DEFAULT_CHAT_FRAME:AddMessage(localization.AUCTIONSCANNING_SCAN_QUEUED, 0.0, 0.8, 0.8);
		end
		
		
	elseif (command == localization.SLASH_FULLREPARSE) then
		local numItemsParsed, numItemsChanged, elapsedTime, usedMemory = self:Parsing_FullReparse();
		
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.LOCALIZATION.REPARSE_REPORT:format(
		    numItemsParsed,
			((numItemsParsed / self.DatabaseSize) * 100),
			self.DatabaseSize,
			numItemsChanged,
			elapsedTime,
			(numItemsParsed / elapsedTime),
			usedMemory,
			(usedMemory / numItemsParsed)
		  ),
		  0.0, 0.8, 0.8
		);
		
		
	elseif (command == localization.SLASH_MERGEDB) then
		if (self:Main_LoadPlugin("LootLink_PremadeDatabase", "PremadeDatabase_DoMerger", true)) then
			self:PremadeDatabase_DoMerger();
		end
		
		
	else
		local chatFrame = DEFAULT_CHAT_FRAME;
		local addMessage = chatFrame.AddMessage;
		
		for index = 1, 8 do
			addMessage(chatFrame, localization["SLASH_COMMAND_HELP_"..index], 0.0, 0.8, 0.8);
		end
	end
end



SlashCmdList["LOOTLINK"] = (function(msg) LootLink:Main_SlashCommandHandler(msg); end);

SLASH_LOOTLINK1 = ("/lootlink");




function LootLink:Main_GetReusableTable(tableNum)
	if (not tableNum) then tableNum = 1; end
	
	local reusableTable = self["ReusableTable"..tableNum];
	
	
	if (not reusableTable) then
		reusableTable = {};
		self["ReusableTable"..tableNum] = reusableTable;
		
	else
		for key in pairs(reusableTable) do
			reusableTable[key] = nil;
		end
	end
	
	
	return reusableTable;
end




function LootLink:Main_LoadPlugin(pluginName, existenceObjName, isAddon)
	if (
	  (isAddon and IsAddOnLoaded(pluginName))
	  or (self[existenceObjName] ~= nil or self.GLOBAL_ENVIRONMENT[existenceObjName] ~= nil)
	) then
		-- Already loaded, return early.
		return true;
	end
	
	
	local funcName = (pluginName.."_Initialize");
	local initFunc = self[funcName];
	
	local success, failureReason;
	
	if (isAddon) then
		collectgarbage("collect");
		
		local currentMemory = collectgarbage("count");
		local extraMemoryNeeded = tonumber(GetAddOnMetadata(pluginName, "X-ExtraMemoryNeeded") or 0);
--		local maxMemory = tonumber(GetCVar("scriptMemory"));
		local maxMemory = 49152
		
		if (currentMemory + extraMemoryNeeded > maxMemory) then
			DEFAULT_CHAT_FRAME:AddMessage(
			  self.LOCALIZATION.PLUGIN_LOAD_FAILED:format(
			    pluginName,
				self.LOCALIZATION.PLUGIN_NOT_ENOUGH_MEMORY:format(extraMemoryNeeded, (maxMemory - currentMemory))
			  ),
			  0.0, 0.8, 0.8
			);
			return false;
		end
		
		success, failureReason = LoadAddOn(pluginName);
	else
		success, failureReason = pcall(initFunc, self);
	end
	
	
	if (success) then
		if (not isAddon) then
			if (self[existenceObjName] ~= nil or self.GLOBAL_ENVIRONMENT[existenceObjName] ~= nil) then
				self[funcName] = nil;
			else
				DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.PLUGIN_LOAD_FAILED:format(pluginName, UNKNOWN), 0.0, 0.8, 0.8);
				return false;
			end
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.PLUGIN_LOAD_FAILED:format(pluginName, failureReason), 0.0, 0.8, 0.8);
		return false;
	end
	
	
	return true;
end




function LootLink:Options_Toggle()
	if (self:Main_LoadPlugin("Options", "OPTIONS_DISPLAY_FRAME")) then
		
		local frame = self.OPTIONS_DISPLAY_FRAME;
		
		if (frame:IsShown()) then
			frame:Hide();
		else
			frame:Show();
		end
		
	end
end




function LootLink:Main_InitConfig()
	if (not LootLink_Config) then
		LootLink_Config = {
			LightMode = nil;
			
			EnableQuickSearch = true;
			AutoFocusQuickSearch = nil;
			
			MobilizeResults = nil;
			
			EnableQuickChatlink = true;
			NoQuickChatlinkForSlashCommands = true;
			NoSuggestionsForQuickChatlink = nil;
			
			EnableSlashLL = true;
			
			TrackLastSeen = true;
			LastSeenOnServer = {};
			
			StoreItemTexturePaths = nil;
			
			MinStoredRarity = 0;
			MinDisplayedRarity = 0;
			
			EnableAutoLinkBot = nil;
			AutoLinkBotMinLength = 4;
			AutoLinkBotWatchedChannels = { GUILD = true; OFFICER = true; WHISPER = true; };
			
			EnableDataSharing = nil;
			TrustedDataSenders = {};
			DataSharingBroadcastChannels = { CHANNEL = true; };
			
			EnableToggleButton = true;
			ToggleButtonX = (UIParent:GetWidth() / 2);
			ToggleButtonY = (UIParent:GetHeight() / 2);
			
			SearchTooltips = true;
			
			-- Configuration for extra tooltip information.
			-- Each tooltip name has a six digit value, representing (in order) vendor price, item level, basic IDs, socket IDs, time last seen, separator, and white text.
			-- A digit being 2 represents that property being enabled for that tooltip, while 1 represents disabled.
			TooltipSettings = {
				GameTooltip = 21111211;
				ItemRefTooltip = 22221212;
				LootLink_ResultsTooltip = 22212212;
				AtlasLootTooltip = 21111211;
			};
			
			Version = self.VERSION;
		};
	end
	
	local config = LootLink_Config;
	self.Config = config;
	
	
	local version = config.Version;
	
	
	if (version < 600) then
		self:DataConversion_FromLootLink5xx_ToLootLink6xx(self.Database, config);
	end
	
	
	if (version < 700) then
		self:DataConversion_FromLootLink6xx_ToLootLink700(self.Database, config);
	end
	
	
	if (version < 720) then
		-- Extend the tooltip numbers to include the setting for white text.
		for tooltipName, tooltipNumbers in pairs(config.TooltipSettings) do
			config.TooltipSettings[tooltipName] = tonumber(tooltipNumbers.."1");
		end
		
		-- Include data for minimum stored rarity and minimum displayed rarity.
		config.MinStoredRarity = 0;
		config.MinDisplayedRarity = 0;
		
	elseif (version == 720 and not config.MinStoredRarity) then
		config.MinStoredRarity = 0;
		config.MinDisplayedRarity = 0;
	end
	
	
	if (version < 770) then
		config.EnableToggleButton = true;
		config.ToggleButtonX = (UIParent:GetWidth() / 2);
		config.ToggleButtonY = (UIParent:GetHeight() / 2);
	end
	
	
	if (version < 780) then
		config.TooltipSettings["AtlasLootTooltip"] = 2111121;
		
		config.AutoLinkBotMinLength = 4;
		
		config.TrustedDataSenders = {};
	end
	
	
	if (version < 782) then
		if (not config.LastSeenOnServer) then
			config.LastSeenOnServer = {};
		end
		
		-- In 7.8.0, accidentally started adding items parsed from AtlasLoot with erroneous suffixes, such as "Gauntlets of Might of Might". Remove the bad suffixes.
		
		local pairs = pairs;
		local next = next;
		local removeSuffixItem = self.ItemTables_RemoveSuffixItem;
		
		local hasOtherSuffixes;
		
		for baseID, itemTable in pairs(self.Database) do
			if (itemTable[3] and itemTable[3] ~= {}) then
				hasOtherSuffixes = true;
				
				for suffix, suffixTable in pairs(itemTable[3]) do
					if (suffixTable[1] == 0) then
						hasOtherSuffixes = removeSuffixItem(self, itemTable, suffix);
					end
				end
				
				if (not hasOtherSuffixes) then
					itemTable[3] = nil;
				end
			end
		end
	end
	
	
	if (version < 790) then
		config.AutoLinkBotWatchedChannels = { GUILD = true; OFFICER = true; WHISPER = true; };
		
		config.DataSharingBroadcastChannels = { CHANNEL = true; };
	end
	
	
	if (version < 800) then
		self:DataConversion_FromLootLink7xx_ToLootLink800(self.Database);
	end
	
	
	if (version < 810) then
		-- Extend the tooltip numbers to include the setting for maximum stack.
		for tooltipName, tooltipNumbers in pairs(config.TooltipSettings) do
			config.TooltipSettings[tooltipName] = tonumber(tooltipNumbers.."1");
		end
	end
	
  --SIZ
  if (version < 829) then
    local pairs = pairs
    local suffixFactor
    
    for baseID, itemTable in pairs(self.Database) do
      if (itemTable[3] and itemTable[3] ~= {}) then
        for suffix, suffixTable in pairs(itemTable[3]) do
          if (suffix < 0) then
            for instanceID, instanceData in pairs(suffixTable) do
              suffixFactor = bit.band(instanceID, 65535);
              if (suffixFactor ~= instanceID) then
                suffixTable[suffixFactor] = instanceData;
                suffixTable[instanceID] = nil;
              end
            end
          end
        end
      end
    end
  end
	
	config.Version = self.VERSION;
	
	
	local realmName = GetRealmName();
	
	if (not config.LastSeenOnServer[realmName]) then
		config.LastSeenOnServer[realmName] = {};
	end
	
	self.LastSeenOnServerForCurrentRealm = config.LastSeenOnServer[realmName];
end




function LootLink:Main_ApplyConfig(config)
	if (not config) then
		config = self.Config;
	end
	
	
	if (config.EnableQuickSearch) then
		if (self:Main_LoadPlugin("QuickSearch", "QuickSearch_OnTextChanged")) then
			self.NUM_RESULT_BUTTONS = 22;
			self.RESULTS_BUTTON_23:Hide();
			LootLink_ResultsFrame_ListScrollFrame:SetHeight(332);
			self.QUICKSEARCH_SEARCH_FRAME:Show();
		end
		
	else
		self.NUM_RESULT_BUTTONS = 23;
		LootLink_ResultsFrame_ListScrollFrame:SetHeight(353);
		self.CurrentQuickSearchIndex = nil;
		
		if (self.QUICKSEARCH_SEARCH_FRAME) then
			self.QUICKSEARCH_SEARCH_FRAME:Hide();
		end
	end
	
	
	if (config.EnableQuickChatlink) then
		self:Main_LoadPlugin("QuickChatlink", "QuickChatlink_ChatEdit_OnChar");
	end
	
	
	if (config.MobilizeResults) then
		UIPanelWindows["LootLink_ResultsFrame"] = nil;
	else
		UIPanelWindows["LootLink_ResultsFrame"] = { area = "left", pushable = 11, whileDead = 1 };
	end
	
	
	if (config.EnableSlashLL) then
		SLASH_LOOTLINK2 = ("/ll");
	else
		SLASH_LOOTLINK2 = nil;
	end
	
	
	if (config.EnableAutoLinkBot) then
		self:Main_LoadPlugin("AutoLinkBot", "AutoLinkBot_ParseRequest");
	end
	
	
	if (not config.TrackLastSeen) then
		local realmName = GetRealmName();
		
		if (next(config.LastSeenOnServer[realmName])) then
			config.LastSeenOnServer[realmName] = {};
			self.LastSeenOnServerForCurrentRealm = config.LastSeenOnServer[realmName];
		end
	end
	
	
	if (config.EnableDataSharing) then
		if (self:Main_LoadPlugin("DataSharing", "DATASHARING_BROADCAST_FRAME")) then
			local realmName = GetRealmName();
			
			if (not config.TrustedDataSenders[realmName]) then
				config.TrustedDataSenders[realmName] = {};
			end
			
			self.TrustedDataSendersForCurrentRealm = config.TrustedDataSenders[realmName];
			
			if (not config.DataSharingBroadcastChannels.CHANNEL) then LeaveChannelByName(self.DATASHARING_CHANNEL_NAME); end
			self.SCRIPTS_FRAME:RegisterEvent("CHAT_MSG_ADDON");
			self.DATASHARING_BROADCAST_FRAME:Show();
		end
		
	elseif (self.DATASHARING_CHANNEL_NAME) then
		LeaveChannelByName(self.DATASHARING_CHANNEL_NAME);
		self.SCRIPTS_FRAME:UnregisterEvent("CHAT_MSG_ADDON");
		self.DATASHARING_BROADCAST_FRAME:Hide();
	end
	
	
	if (config.EnableToggleButton) then
		if (self:Main_LoadPlugin("ToggleButton", "TOGGLEBUTTON_BUTTON")) then
			self.TOGGLEBUTTON_BUTTON:Show();
		end
		
	elseif (self.TOGGLEBUTTON_BUTTON) then
		self.TOGGLEBUTTON_BUTTON:Hide();
	end
end




function LootLink:Main_InitDatabase()
	if (not LootLink_Database) then
		LootLink_Database = {};
	end
	
	local database = LootLink_Database;
	
	self.Database = database;
	
	
	if (ItemLinks) then
		self:DataConversion_MergeTeloLootLink350_WithLootLinkCurrent(ItemLinks, database);
		ItemLinks = nil;
	end
	
	
	local pairs = pairs;
	local type = type;
	local tonumber = tonumber;
	local setmetatable = setmetatable;
	
	local baseMetatable = self.BASEITEM_METATABLE;
	
	local databaseSize = 0;
	
	
	for baseID, itemTable in pairs(database) do
		if (type(baseID) ~= "number") then
			database[baseID] = nil;
			
			baseID = tonumber(baseID);
			if (baseID) then
				database[baseID] = itemTable;
			end
		end
		
		
		if (baseID) then
			itemTable[-1] = baseID; -- For self-reference later. Very useful.
			
			if (not itemTable[1]) then itemTable[1] = UNKNOWN; end
			if (not itemTable[2]) then itemTable[2] = 7; end
			
			if (itemTable[3] and itemTable[3] ~= {}) then
				for suffixID, suffixTable in pairs(itemTable[3]) do
					
					if (type(suffixID) ~= "number") then
						itemTable[3][suffixID] = nil;
						
					elseif (suffixID < 0) then
						if (type(suffixTable) ~= "table") then
							itemTable[suffixID] = nil;
						else
							for instanceID, instanceData in pairs(suffixTable) do
								databaseSize = (databaseSize + 1);
							end
						end
						
					else
						databaseSize = (databaseSize + 1);
					end
					
				end
			end
			
			databaseSize = (databaseSize + 1);
			
			setmetatable(itemTable, baseMetatable);
		end
	end
	
	
	setmetatable(self.Database, self.DATABASE_METATABLE);
	
	self.DatabaseSize = databaseSize;
	
	
	self:Compatibility_Initialize();
end

