

local LootLink = LootLink;




function LootLink:Matching_LimitMatchesToNum(linksTable, maxMatches)
	local numEntries = 0;
	local isGreater = nil;

	for baseID, _ in pairs(linksTable) do

		if (isGreater) then
			linksTable[baseID] = nil;

		else
			numEntries = (numEntries + 1);

			if (numEntries > maxMatches) then
				isGreater = true;
				linksTable[baseID] = nil;
			end
		end

	end

	return linksTable;
end




function LootLink:Matching_ByCharsInName(linksTable, doExpand, maxMatches, itemName, plain, skipSuffixes)
	local stringfind = self.STRING_LIBRARY.find;
	local stringlower = self.STRING_LIBRARY.lower;
	local pairs = pairs;
	
	local suffixIDsToNameKeys = self.SUFFIX_IDS_TO_NAME_KEYS;
	local suffixNameKeysToNames = self.SUFFIX_NAME_KEYS_TO_NAMES;
	
	local numMatches = 0;
	
	local currentName;
	local acted;
	
	itemName = stringlower(itemName);
	
	
	for baseID, itemTable in pairs((doExpand and self.Database) or linksTable) do
		currentName = stringlower(itemTable[1]);
		
		if (stringfind(currentName, itemName, 1, plain)) then
			if (doExpand) then
				linksTable[baseID] = itemTable;
			end
			
			numMatches = (numMatches + 1);
			
			if (numMatches == maxMatches) then
				return linksTable, numMatches;
			end
			
			
		elseif (not skipSuffixes and itemTable[3]) then
			currentName = (currentName.." ");
			
			acted = nil;
			
			for suffixID, suffixData in pairs(itemTable[3]) do
				if (not acted) then
					
					if (stringfind(currentName..stringlower(suffixNameKeysToNames[suffixIDsToNameKeys[suffixID]] or ""), itemName, 1, plain)) then
						if (doExpand) then
							linksTable[baseID] = itemTable;
						end
						acted = true;
					end
					
				end
			end
			
			if (acted) then
				numMatches = (numMatches + 1);
				if (numMatches == maxMatches) then
					return linksTable, numMatches;
				end
				
			elseif (not doExpand) then
				linksTable[baseID] = nil;
			end
			
			
		elseif (not doExpand) then
			linksTable[baseID] = nil;
		end
	end
	
	
	return linksTable, numMatches;
end




function LootLink:Matching_ByLevel(linksTable, doExpand, maxMatches, level)
	local numMatches = 0;
	
	level = tonumber(level);
	
	
	for baseID, itemTable in pairs((doExpand and self.Database) or linksTable) do
		if (itemTable[10] == level) then
			if (doExpand) then
				linksTable[baseID] = itemTable;
			end
			
			numMatches = (numMatches + 1);
			
			if (numMatches == maxMatches) then
				return linksTable, numMatches;
			end
			
			
		elseif (not doExpand) then
			linksTable[baseID] = nil;
		end
	end
	
	
	return linksTable, numMatches;
end




function LootLink:Matching_ByCached(linksTable, doExpand, maxMatches)
	local getItemInfo = GetItemInfo;
	
	local numMatches = 0;
	
	
	for baseID, itemTable in pairs((doExpand and self.Database) or linksTable) do
		if (getItemInfo(baseID)) then
			if (doExpand) then
				linksTable[baseID] = itemTable;
			end
			
			numMatches = (numMatches + 1);
			
			if (numMatches == maxMatches) then
				return linksTable, numMatches;
			end
			
			
		elseif (not doExpand) then
			linksTable[baseID] = nil;
		end
	end
	
	
	return linksTable, numMatches;
end




function LootLink:Matching_GetItemTableAndSuffixIDByName(tableToSearch, itemName)
	local pairs = pairs;
	local stringlower = self.STRING_LIBRARY.lower;
	local stringsub = self.STRING_LIBRARY.sub;
	local stringlen = self.STRING_LIBRARY.len;
	
	local suffixIDsToNameKeys = self.SUFFIX_IDS_TO_NAME_KEYS;
	local suffixNameKeysToNames = self.SUFFIX_NAME_KEYS_TO_NAMES;
	
	local currentName;
	
	itemName = stringlower(itemName);
	
	
	for baseID, itemTable in pairs(tableToSearch or self.Database) do
		currentName = stringlower(itemTable[1]);
		
		-- If item's name, "foo", equals "foo" from input text "foo of bar"...
		if (currentName == stringsub(itemName, 1, stringlen(currentName))) then
			
			if (currentName == itemName) then
				return itemTable, nil;
				
				
			elseif (itemTable[3]) then
				currentName = (currentName.." ");
				
				for suffixID in pairs(itemTable[3]) do
					if ((currentName..stringlower(suffixNameKeysToNames[suffixIDsToNameKeys[suffixID]])) == itemName) then
						return itemTable, suffixID;
					end
				end
			end
			
		end
	end
	
	
	return nil, nil;
end




function LootLink:Matching_GetHyperlinkByItemTableAndSuffixID(itemTable, suffixID, instanceID)
	local itemName = itemTable[1];
	
	if (suffixID and suffixID ~= 0) then
		itemName = (itemName.." "..self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]]);
		
		if (suffixID < 0 and (not instanceID or instanceID == 0)) then
			instanceID = next(itemTable[3][suffixID]);
		end
	end
	
	return ("%s|Hitem:%d:0:0:0:0:0:%d:%d|h[%s]|h|r"):format(
	  ITEM_QUALITY_COLORS[(itemTable[2] < 7 and itemTable[2]) or 1].hex,
	  itemTable[-1],
	  (suffixID or 0),
	  (instanceID or 0),
	  itemName
	);
end

