

local LootLink = LootLink;




LootLink.Original_GameTooltip_SetHyperlink = GameTooltip.SetHyperlink;



LootLink.Original_TradeSkillFrame_LoadUI = TradeSkillFrame_LoadUI;


LootLink.Original_TradeSkillFrame_Show = nil;



LootLink.UnitInventoryMouseoverParseDelays = {};




function LootLink:Parsing_ParseFromTable(linksTable)
	local stringmatch = self.STRING_LIBRARY.match;
	local getItemInfo = GetItemInfo;
	local stringtrim = self.STRING_LIBRARY.trim;
	local stringgsub = self.STRING_LIBRARY.gsub;
	local getRarityNumberForHex = self.Parsing_GetRarityNumberForHex;
	local tonumber = tonumber;
	local addBaseItem = self.ItemTables_AddBaseItem;
	local addSuffixItem = self.ItemTables_AddSuffixItem;
	
	local database = self.Database;
	local suffixIDsToNameKeys = self.SUFFIX_IDS_TO_NAME_KEYS;
	local suffixNameKeysToNames = self.SUFFIX_NAME_KEYS_TO_NAMES;
	local trackLastSeen = self.Config.TrackLastSeen;
	local lastSeenTable = self.LastSeenOnServerForCurrentRealm;
	
	local currentTime = time();
	
	local color, baseID, suffixID, instanceID, itemName;
	local suffix;
	local currentBaseTable;
	
	
	for index, link in ipairs(linksTable) do
		color, baseID, suffixID, instanceID, itemName = stringmatch(link, "^(|c.-)|Hitem:(.-):.-:.-:.-:.-:.-:(.-):(.-)|h%[(.-)%]|h|r$");
		
		if (color) then
			realName, _, rarityNumber = getItemInfo("item:"..baseID..":0:0:0:0:0"..suffixID..":0");
			
			if (realName) then
				itemName = realName;
				color = rarityNumber;
			else
				color = getRarityNumberForHex(self, color);
			end
			
			baseID = tonumber(baseID);
			suffixID = tonumber(suffixID);
			
			
			if (suffixID ~= 0) then
				suffix = suffixNameKeysToNames[suffixIDsToNameKeys[suffixID]];
				
				if (suffix) then
					itemName = stringtrim((stringgsub(itemName, suffix, "", 1)));
				end
			end
			
			
			currentBaseTable = (database[baseID] or addBaseItem(self, baseID, itemName, color));
			
			if (currentBaseTable and suffixID ~= 0 and (not currentBaseTable[3] or not currentBaseTable[3][suffixID])) then
				addSuffixItem(self, currentBaseTable, suffixID, tonumber(instanceID));
			end
			
			
			if (trackLastSeen) then
				lastSeenTable[baseID] = currentTime;
			end
		end
		
		linksTable[link] = nil;
	end
end




function LootLink:Parsing_ParseFromString(linksText)
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	
	for link in linksText:gmatch("|c.-|Hitem:.-|h%[.-%]|h|r") do
		tableinsert(linksTable, link);
	end
	
	if (#linksTable > 0) then
		self:Parsing_ParseFromTable(linksTable);
	end
end




function LootLink:Parsing_ParseFromHyperlink(hyperlink)
	local name, itemString, rarity = GetItemInfo(hyperlink);
	
	if (name) then
		local linksTable = self:Main_GetReusableTable(1);
		self.TABLE_LIBRARY.insert(linksTable, itemString);
		
		self:Parsing_ParseFromTable(linksTable);
	end
end




function LootLink:Parsing_GetRarityNumberForHex(hexColor)
	for index, colorTable in pairs(ITEM_QUALITY_COLORS) do
		if (colorTable.hex == hexColor) then
			return index;
		end
	end
	
	return 7; -- Unknown
end




function LootLink:Parsing_ParseInventory(unit)
	local delays = self.UnitInventoryMouseoverParseDelays;
	local currTime = time();
	local name = UnitName(unit);
	local unitTime = delays[name];
	
	if ((unitTime ~= nil and unitTime < currTime) or CanInspect(unit) == nil) then
		return;
	end
	
	delays[name] = (currTime + self.MOUSEOVER_PARSING_DELAY);
	
	
	local getInventoryItemLink = GetInventoryItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	local linksTable = self:Main_GetReusableTable(1);
	local link;
	
	for slot = 23, 1, -1 do
		link = getInventoryItemLink(unit, slot);
		
		if (link) then
			tableinsert(linksTable, link);
		end
	end

	self:Parsing_ParseFromTable(linksTable);
end

function LootLink:Parsing_ParseInbox()
	local GetInboxHeaderInfo = GetInboxHeaderInfo;
	local GetInboxItemLink = GetInboxItemLink
	local tableinsert = self.TABLE_LIBRARY.insert;
	local linksTable = self:Main_GetReusableTable(1);

	local numInboxItems = GetInboxNumItems()
	if( numInboxItems > 0 ) then
		local link
		local hasItem
		local index, attachSlot
		for index = 1, numInboxItems do
			hasItem = select(8, GetInboxHeaderInfo(index))
			if hasItem then
				for attachSlot=1, ATTACHMENTS_MAX_SEND do
					link = GetInboxItemLink(index, attachSlot)
					if (link) then
						tableinsert(linksTable, link);
					end
				end -- for each attachment
			end	--if hasItem
		end
	end

	self:Parsing_ParseFromTable(linksTable);
end



function LootLink:Parsing_ParseCurrentAuctionList()
	local getAuctionItemLink = GetAuctionItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	local link;
	
--	DEFAULT_CHAT_FRAME:AddMessage("Number of Items in the list: "..GetNumAuctionItems("list"))
	
	
	for auctionIndex = GetNumAuctionItems("list"), 1, -1 do
		link = getAuctionItemLink("list", auctionIndex);
		
		if (link) then
			tableinsert(linksTable, link);
		end
	end
	
	
	self:Parsing_ParseFromTable(linksTable);
end




function LootLink:Parsing_ParseMerchant()
	local getMerchantItemLink = GetMerchantItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	
	for slot = GetMerchantNumItems(), 1, -1 do
		link = getMerchantItemLink(slot);
		
		if (link) then
			tableinsert(linksTable, link);
		end
	end
	
	
	self:Parsing_ParseFromTable(linksTable);
end



function LootLink:Parsing_ParseSellPrices()
	local tooltip = LootLink_ParsingTooltip;
	
	
	local getContainerNumSlots = GetContainerNumSlots;
	local getContainerItemLink = GetContainerItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	local stringmatch = self.STRING_LIBRARY.match;
	local tonumber = tonumber;
	local getItemInfo = GetItemInfo;
	local setBagItem = tooltip.SetBagItem;
	local getContainerItemInfo = GetContainerItemInfo;
	
	local database = self.Database;
	local linksTable = self:Main_GetReusableTable(1); -- We're iterating over all of the bag items anyway, so why not parse them now too?
	
	local link;
	local baseID;
	local itemTable;
	local baseName, baseLink, rarityNumber;
	local hasCooldown, repairCost;
	local texture, quantity;
	
	
	for bag = 4, 0, -1 do
		for slot = getContainerNumSlots(bag), 1, -1 do
			link = getContainerItemLink(bag, slot);
			
			if (link) then
				tableinsert(linksTable, link);
				
				
				baseID = stringmatch(link, "item:(%d+):", 13);
				itemTable = database[tonumber(baseID)];
				
				if (not itemTable) then
					baseName, baseLink, rarityNumber = getItemInfo(baseID);
					itemTable = self:ItemTables_AddBaseItem(baseID, baseName, rarityNumber);
				end
				
				
				if (itemTable) then
					self.CurrentScanVendorValue = false; -- Initially set this to false to indicate that the item is unsellable. If there is a monetary value, this value will be overwritten.
					
					hasCooldown, repairCost = setBagItem(tooltip, bag, slot);
					
					if (self.CurrentScanVendorValue == false) then
						itemTable[6] = false;
						
					-- Check repair cost to avoid storing damaged items, unless there is no previous price associated with the item.
					elseif (repairCost == 0 or not itemTable[6]) then
						texture, quantity = getContainerItemInfo(bag, slot);
						itemTable[6] = (self.CurrentScanVendorValue / quantity);
					end
					
				end
			end
			
		end
	end
	
	
	self.CurrentScanVendorValue = nil;
	
	
	self:Parsing_ParseFromTable(linksTable);
end


LootLink.PARSING_TOOLTIP:SetScript("OnTooltipAddMoney", (function(self, money) self.LOOTLINK.CurrentScanVendorValue = money; end));




function LootLink:Parsing_ParseBank()
	local getContainerNumSlots = GetContainerNumSlots;
	local getContainerItemLink = GetContainerItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	local link;
	
	
	for bag = 10, 5, -1 do
		for slot = getContainerNumSlots(bag), 1, -1 do
			link = getContainerItemLink(bag, slot);
			
			if (link) then
				tableinsert(linksTable, link);
			end
		end
	end
	
	
	for slot = 24, 1, -1 do
		link = getContainerItemLink(-1, slot);
		
		if (link) then
			tableinsert(linksTable, link);
		end
	end
	
	
	self:Parsing_ParseFromTable(linksTable);
end




function LootLink:Parsing_ParseTradeWindow()
	local getTradeTargetItemLink = GetTradeTargetItemLink;
	local getTradePlayerItemLink = GetTradePlayerItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	local link;
	
	
	for slot = 7, 1, -1 do
		link = getTradeTargetItemLink(slot);
		
		if (link) then
			tableinsert(linksTable, link);
		end
		
		link = getTradePlayerItemLink(slot);

		if (link) then
			tableinsert(linksTable, link);
		end
	end
	
	
	self:Parsing_ParseFromTable(linksTable);
end




function LootLink:Parsing_ParseBags()
	local getContainerNumSlots = GetContainerNumSlots;
	local getContainerItemLink = GetContainerItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	local link;
	
	
	for bag = 4, 0, -1 do
		for slot = getContainerNumSlots(bag), 1, -1 do
			link = getContainerItemLink(bag, slot);
			
			if (link) then
				tableinsert(linksTable, link);
			end
		end
	end
	
	
	self:Parsing_ParseFromTable(linksTable);
end




GameTooltip.SetHyperlink = function(self, hyperlink)
	LootLink.Original_GameTooltip_SetHyperlink(self, hyperlink);
	
	if (hyperlink) then
		LootLink:Parsing_ParseFromHyperlink(hyperlink);
	end
end




function LootLink:Parsing_ParseTradeSkills()
	local getTradeSkillItemLink = GetTradeSkillItemLink;
	local getTradeSkillNumReagents = GetTradeSkillNumReagents;
	local getTradeSkillReagentItemLink = GetTradeSkillReagentItemLink;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local linksTable = self:Main_GetReusableTable(1);
	
	local link;
	
	
	for index = GetNumTradeSkills(), 1, -1 do
		link = getTradeSkillItemLink(index);
		
		if (link) then
			tableinsert(linksTable, link);
			
			for subIndex = getTradeSkillNumReagents(index), 1, -1 do
				link = getTradeSkillReagentItemLink(index, subIndex);
				if (link) then
					tableinsert(linksTable, link);
				end
			end
			
		end
	end
	
	
	self:Parsing_ParseFromTable(linksTable);
end



function LootLink.PostHook_TradeSkillFrame_Show()
	LootLink.Original_TradeSkillFrame_Show();
	LootLink:Parsing_ParseTradeSkills();
end



TradeSkillFrame_LoadUI = function()
	LootLink.Original_TradeSkillFrame_LoadUI();
	
	if (not LootLink.Original_TradeSkillFrame_Show) then
		LootLink.Original_TradeSkillFrame_Show = TradeSkillFrame_Show;
		TradeSkillFrame_Show = LootLink.PostHook_TradeSkillFrame_Show;
	end
end




function LootLink:Parsing_FullReparse(database)
	local startTime = GetTime();
	local startMemory = collectgarbage("count");
	
	
	self:Parsing_ParseInventory("player");
	
	self:Parsing_ParseBags();
	
	
	local pairs = pairs;
	local type = type;
	local getItemInfo = GetItemInfo;
	local getBaseTableHash = self.ItemTables_GetBaseTableHash;
	local populate = self.ItemTables_Populate;
	
	local numItems = 0;
	local numChanges = 0;
	
	local staticName, baseHash, suffixHash;
	
	
	for baseID, itemTable in pairs(database or self.Database) do
		staticName = getItemInfo(baseID);
		
		if (staticName) then
			numItems = (numItems + 1);
			baseHash = getBaseTableHash(self, itemTable);
			
			
			if (itemTable[1] ~= staticName) then
				DEFAULT_CHAT_FRAME:AddMessage(
				  self.LOCALIZATION.REPARSE_NAME_MISMATCH:format((itemTable[1] or self.LOCALIZATION.GENERIC_NONE), staticName),
				  0.0, 0.8, 0.8
				);
				itemTable[1] = staticName;
			end
			
			
			if (itemTable[3]) then
				for suffixID, suffixData in pairs(itemTable[3]) do
					if (suffixID < 0) then
						if (type(suffixData) == "table") then
							for instanceID, instanceData in pairs(suffixData) do
								numItems = (numItems + 1);
								suffixHash = instanceData;

                instanceID = bit.band(instanceID, 65535)  --Siz

								populate(self, itemTable, suffixID, instanceID);
								
								if (suffixData[instanceID] ~= instanceData) then
									numChanges = (numChanges + 1);
								end
							end
						end
						
					else
						numItems = (numItems + 1);
						suffixHash = suffixData;
						
						populate(self, itemTable, suffixID);
						
						if (itemTable[3][suffixID] ~= suffixData) then
							numChanges = (numChanges + 1);
						end
					end
				end
				
				
			else
				populate(self, itemTable);
			end
			
			
			if (getBaseTableHash(self, itemTable) ~= baseHash) then
				numChanges = (numChanges + 1);
			end
			
			
		else
			if (itemTable[5] and next(itemTable[5]) == nil) then
				itemTable[5] = nil;
			end
			
			if (itemTable[3]) then
				for suffixID, suffixData in pairs(itemTable[3]) do
					if (suffixID < 0) then
						if (type(suffixData) == "table") then
							if (next(suffixData) == nil) then
								itemTable[3][suffixID] = nil;
								
							else
								for instanceID, instanceData in pairs(suffixData) do
									if (type(instanceData) == "table" and (next(instanceData) == nil or (instanceData[0] and next(instanceData, 0) == nil))) then
										suffixData[instanceID] = (instanceData[0] or false);
									end
								end
							end
						end
						
					elseif (type(suffixData) == "table" and next(suffixData) == nil) then
						itemTable[3][suffixID] = false;
					end
				end
			end
		end
	end
	
	
	-- Return the statistics of the operation.
	return numItems, numChanges, (GetTime() - startTime), (collectgarbage("count") - startMemory);
end

