

local LootLink = LootLink;




LootLink.ResultsList = {};


-- The number of items, including subitems, which matched the search criteria. This number is used to update the title text, and only changes when a new search is performed.
LootLink.RealResultsSize = 0;

-- The number of clickable entries in the results list. This number expands and shrinks as items with subitems have the view of those subitems toggled on and off.
LootLink.ScrollableResultsSize = 0;


LootLink.LastViewedDatabaseSize = 0;



LootLink_ResultsFrame_ListScrollFrame.Hide = (function() end);




function LootLink:Results_Toggle()
	local frame = self.RESULTS_FRAME;
	
	if (frame:IsShown()) then
		HideUIPanel(frame);
	else
		ShowUIPanel(frame);
	end
end




function LootLink:Results_OnShow()
	PlaySound("igMainMenuOpen");
	
	
	if (self.LastViewedDatabaseSize ~= self.DatabaseSize) then
		self.LastViewedDatabaseSize = self.DatabaseSize;
		
		if (not next(self.SearchParameters)) then
			self:Results_FullRebuild();
		else
			self:Results_UpdateTitle(true);
		end
	end
	
	self:Results_UpdateCurrentPage();
	
	
	if (self.Config.EnableQuickSearch and self.Config.AutoFocusQuickSearch) then
		self.QUICKSEARCH_SEARCH_EB:SetFocus();
	end
end


LootLink.RESULTS_FRAME:SetScript("OnShow", (function(self) self.LOOTLINK:Results_OnShow(); end));




function LootLink:Results_OnHide()
	PlaySound("igMainMenuClose");
	
	self.SEARCH_FRAME:Hide();
	
	if (self.OPTIONS_FRAME) then
		self.OPTIONS_FRAME:Hide();
	end
end


LootLink.RESULTS_FRAME:SetScript("OnHide", (function(self) self.LOOTLINK:Results_OnHide(); end));




function LootLink:Results_FullRebuild()
	self:Results_ClearAndRepopulateList(self.SearchParameters);
	
	if (self.CurrentQuickSearchIndex) then
		self.CurrentQuickSearchIndex = 1;
	end
	
	self:Sorting_SortBySortID(self.SelectedSortID, self.SortIsReversed);
	
	self:Results_UpdateTitle();
	
	self:Results_UpdateCurrentPage();
end




function LootLink:Results_UpdateCurrentPage()
	FauxScrollFrame_Update(LootLink_ResultsFrame_ListScrollFrame, self.ScrollableResultsSize, self.NUM_RESULT_BUTTONS, self.RESULT_BUTTON_HEIGHT);
	
	
	local tooltip = self.RESULTS_TOOLTIP;
	
	local type = type;
	local isOwned = tooltip.IsOwned;
	
	local resultsList = self.ResultsList;
	local rarityData = ITEM_QUALITY_COLORS;
	local suffixIDsToNameKeys = self.SUFFIX_IDS_TO_NAME_KEYS;
	local suffixNameKeysToNames = self.SUFFIX_NAME_KEYS_TO_NAMES;
	
	local offset = LootLink_ResultsFrame_ListScrollFrame.offset;
	
	local button, currentEntry, lastBaseItem, suffixID, instanceID, colorTable;
	
	
	for index = 1, self.NUM_RESULT_BUTTONS do
		button = self["RESULTS_BUTTON_"..index];
		currentEntry = resultsList[index + offset];
		
		if (currentEntry) then
			
			if (type(currentEntry) == "table") then
				lastBaseItem = currentEntry;
				
				if (currentEntry[-2] == 0) then
					button:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-UP");
					button:SetText(currentEntry[1].." ("..currentEntry[-3]..")");
					
				elseif (currentEntry[-2] == 1) then
					button:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-UP");
					button:SetText(currentEntry[1].." ("..currentEntry[-3]..")");
					
				else
					button:SetNormalTexture("");
					button:SetText(currentEntry[1]);
				end
				
				button.ItemTable = currentEntry;
				button.SuffixID = nil;
				button.InstanceID = nil;
				
				
			else
				if (not lastBaseItem) then
					-- If there is no lastBaseItem, then this suffix is at the top of the page, so no base item has been indexed yet. Search backwards through the list to find one.
					local searchIndex = (index + offset);
					
					while (searchIndex >= 1 and not lastBaseItem) do
						if (type(resultsList[searchIndex]) == "table") then
							lastBaseItem = resultsList[searchIndex];
						end
						
						searchIndex = (searchIndex - 1);
					end
				end
				
				
				if (type(currentEntry) == "string") then
					suffixID, instanceID = currentEntry:match("^(.-) (.-)$");
				else
					suffixID = currentEntry;
					instanceID = nil;
				end
				
				suffixID = tonumber(suffixID);
				if (instanceID) then instanceID = tonumber(instanceID); end
				
				button:SetText("    "..suffixNameKeysToNames[suffixIDsToNameKeys[suffixID]]);
				button:SetNormalTexture("");
				
				button.ItemTable = lastBaseItem;
				button.SuffixID = suffixID;
				button.InstanceID = instanceID;
			end
			
			
			colorTable = rarityData[button.ItemTable[2]];
			
			if (colorTable) then
				button:SetTextColor(colorTable.r, colorTable.g, colorTable.b);
				button:SetHighlightTextColor(colorTable.r, colorTable.g, colorTable.b);
			else
				button:SetTextColor(0.0, 0.8, 0.8);
				button:SetHighlightTextColor(0.0, 0.8, 0.8);
			end
			
			button:Show();
			
			if (isOwned(tooltip, button)) then
				self:Results_ItemButton_OnEnter(button);
			end
			
			
		else
			button.ItemTable = nil;
			button.SuffixID = nil;
			button.InstanceID = nil;
			button:Hide();
		end
	end
end



local function LootLink_Results_UpdateCurrentPage() LootLink:Results_UpdateCurrentPage(); end

LootLink_ResultsFrame_ListScrollFrame:SetScript("OnVerticalScroll", function(self)
	FauxScrollFrame_OnVerticalScroll(LootLink.RESULT_BUTTON_HEIGHT, LootLink_Results_UpdateCurrentPage); 
end);




function LootLink:Results_ClearAndRepopulateList(searchParams)
	LootLink_ResultsFrame_ListScrollFrame.offset = 0;
	LootLink_ResultsFrame_ListScrollFrameScrollBar:SetValue(0);
	
	
	local resultsList = self.ResultsList;
	
	for index in pairs(resultsList) do
		resultsList[index] = nil;
	end
	
	
	if (not searchParams or not next(searchParams)) then
		self.ScrollableResultsSize, self.RealResultsSize = self:Results_PopulateList_AllItems(self.Database, self.ResultsList);
	else
		self.ScrollableResultsSize, self.RealResultsSize = self:Results_PopulateList_BySearchParameters(self.Database, self.ResultsList, searchParams);
	end
end




function LootLink:Results_PopulateList_AllItems(database, resultsList)
	local pairs = pairs;
	local tableinsert = self.TABLE_LIBRARY.insert;
	local type = type;
	
	local minRarity = self.Config.MinDisplayedRarity;
	
	local numItems = 0;
	local numSuffixes;
	
	
	for baseID, itemTable in pairs(database) do
		if (itemTable[2] >= minRarity) then
			
			numItems = (numItems + 1);
			tableinsert(resultsList, itemTable);
			
			
			if (itemTable[3] and itemTable[3] ~= {}) then
				itemTable[-2] = 0; -- Set the tracking flag to "collapsed" status.
				
				numSuffixes = 0;
				
				for suffixID, suffixData in pairs(itemTable[3]) do
					if (suffixID < 0) then
						if (type(suffixData) == "table") then
							for instanceID, instanceData in pairs(suffixData) do
								numSuffixes = (numSuffixes + 1);
								if (type(instanceData) == "table") then
									instanceData[0] = true;
								else
									suffixData[instanceID] = true;
								end
							end
						end
						
					else
						numSuffixes = (numSuffixes + 1);
						
						if (type(suffixData) == "table") then
							suffixData[0] = true;
						else
							itemTable[3][suffixID] = true;
						end
					end
				end
				
				itemTable[-3] = numSuffixes;
				numItems = (numItems + numSuffixes);
				
				
			else
				itemTable[-2] = nil; -- Not a suffixed item, don't need to track toggled status.
				itemTable[-3] = nil; -- Not a suffixed item, don't need to track number of suffixes.
				
			end
			
		end
	end
	
	
	return #resultsList, numItems;
end




function LootLink:Results_PopulateList_BySearchParameters(database, resultsList, searchParams)
	local pairs = pairs;
	local stringfind = self.STRING_LIBRARY.find;
	local stringlower = self.STRING_LIBRARY.lower;
	local getConcatenatedFullText = self.ItemTables_GetConcatenatedFullText;
	local type = type;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local suffixIDsToNameKeys = self.SUFFIX_IDS_TO_NAME_KEYS;
	local suffixNameKeysToNames = self.SUFFIX_NAME_KEYS_TO_NAMES;
	
	
	local text = (searchParams.text and stringlower(searchParams.text));
	local name = (searchParams.name and stringlower(searchParams.name));
	
	local rarity = searchParams.rarity;
	local bindType = searchParams.bindType;
	
	local minEquipLevel = searchParams.minEquipLevel;
	local maxEquipLevel = searchParams.maxEquipLevel;
	local minItemLevel = searchParams.minItemLevel;
	local maxItemLevel = searchParams.maxItemLevel;
	
	local equipLocation = searchParams.equipLocation;
	local unique = searchParams.unique;
	
	local basetype = searchParams.basetype;
	local subtype = searchParams.subtype;
	
	
	local plain = (not IsControlKeyDown());
	local minRarity = (self.Config.MinDisplayedRarity > 0 and self.Config.MinDisplayedRarity);
	
	local numItems = 0;
	
	local currentName;
	local numMatchingSuffixes;
	
	
	for baseID, itemTable in pairs(database) do
		if (
		  (not minRarity or itemTable[2] >= minRarity)
		  and (not rarity or itemTable[2] == rarity)
		  and (not bindType or (bindType == -1 and not itemTable[4]) or itemTable[4] == bindType)
		  and (not basetype or itemTable[8] == basetype)
		  and (not subtype or itemTable[9] == subtype)
		  and (not equipLocation or (equipLocation == -1 and not itemTable[7]) or itemTable[7] == equipLocation)
		  and (not unique or itemTable[11])
		  and (not minEquipLevel or (itemTable[10] or 1) >= minEquipLevel)
		  and (not maxEquipLevel or (itemTable[10] or 70) <= maxEquipLevel)
		  and (not minItemLevel or (itemTable[12] or 0) >= minItemLevel)
		  and (not maxItemLevel or (itemTable[12] or 0) <= maxItemLevel)
		) then
			
			
			if (
			  (not name and not text)
			  or (
			    (not name or stringfind(stringlower(itemTable[1]), name, 1, plain))
			    and (not text or stringfind(stringlower(getConcatenatedFullText(self, itemTable)), text, 1, plain))
			  )
			) then
				numItems = (numItems + 1);
				
				
				if (itemTable[3] and itemTable[3] ~= {}) then
					numMatchingSuffixes = 0;
					
					for suffixID, suffixData in pairs(itemTable[3]) do
						if (suffixID < 0) then
							if (type(suffixData) == "table") then
								for instanceID, instanceData in pairs(suffixData) do
									numMatchingSuffixes = (numMatchingSuffixes + 1);
									
									if (type(instanceData) == "table") then
										instanceData[0] = true;
									else
										suffixData[instanceID] = true;
									end
								end
							end
						else
							numMatchingSuffixes = (numMatchingSuffixes + 1);
							
							if (type(suffixData) == "table") then
								suffixData[0] = true;
							else
								itemTable[3][suffixID] = true;
							end
						end
					end
					
					itemTable[-2] = 0;
					itemTable[-3] = numMatchingSuffixes;
					
					
				else
					itemTable[-2] = nil;
					itemTable[-3] = nil;
				end
				
				
				tableinsert(resultsList, itemTable);
				
				
			elseif (itemTable[3] and itemTable[3] ~= {}) then
				numMatchingSuffixes = 0;
				
				
				if (name) then currentName = (stringlower(itemTable[1]).." "); end
				
				for suffixID, suffixData in pairs(itemTable[3]) do
					suffixName = suffixNameKeysToNames[suffixIDsToNameKeys[suffixID]];
					
					if (suffixName and (not name or stringfind((currentName..stringlower(suffixName)), name, 1, plain))) then
						
						if (suffixID < 0) then
							for instanceID, instanceData in pairs(suffixData) do
								if (not text or stringfind(stringlower(getConcatenatedFullText(self, itemTable, suffixID, instanceID, suffixName)), text, 1, plain)) then
									numMatchingSuffixes = (numMatchingSuffixes + 1);
									
									if (type(instanceData) == "table") then
										instanceData[0] = true;
									else
										suffixData[instanceID] = true;
									end
									
								else
									if (type(instanceData) == "table") then
										instanceData[0] = nil;
									else
										suffixData[instanceID] = false;
									end
								end
							end
							
							
						else
							if (not text or stringfind(stringlower(getConcatenatedFullText(self, itemTable, suffixID, nil, suffixName)), text, 1, plain)) then
								numMatchingSuffixes = (numMatchingSuffixes + 1);
								
								if (type(suffixData) == "table") then
									suffixData[0] = true;
								else
									itemTable[3][suffixID] = true;
								end
								
							else
								if (type(suffixData) == "table") then
									suffixData[0] = nil;
								else
									itemTable[3][suffixID] = false;
								end
							end
						end
						
						
					else
						if (suffixID < 0) then
							for instanceID, instanceData in pairs(suffixData) do
								if (type(instanceData) == "table") then
									instanceData[0] = nil;
								else
									suffixData[instanceID] = false;
								end
							end
							
						else
							if (type(suffixData) == "table") then
								suffixData[0] = nil;
							else
								itemTable[3][suffixID] = false;
							end
						end
						
					end
				end
				
				
				if (numMatchingSuffixes > 0) then
					itemTable[-2] = 0; -- Set the tracking flag to "collapsed" status.
					itemTable[-3] = numMatchingSuffixes;
					numItems = (numItems + numMatchingSuffixes + 1);
					
					tableinsert(resultsList, itemTable);
					
				else
					itemTable[-2] = nil;
					itemTable[-3] = nil;
				end
				
				
			else
				itemTable[-2] = nil;
				itemTable[-3] = nil;
				
			end
		end
	end
	
	
	return #resultsList, numItems;
end




function LootLink:Results_UpdateTitle(activateNewItemWarning)
	LootLink_ResultsFrame_TitleText:SetText(
	  self.LOCALIZATION.RESULTS_TITLE_FORMAT:format(
	    self.RealResultsSize,
		-- 12.345% if not all results are showing, or 100% if they are
	    ((self.RealResultsSize ~= self.DatabaseSize and ("%5.3f"):format((self.RealResultsSize /  self.DatabaseSize) * 100)) or 100),
	    self.DatabaseSize
	  )
	);
	
	
	if (activateNewItemWarning) then
		LootLink_ResultsFrame_TitleText:SetTextColor(1.0, 1.0, 0.0);
		
		if (LootLink_ToggleButton_ButtonText) then
			LootLink_ToggleButton_ButtonText:SetTextColor(1.0, 1.0, 0.0);
		end
		
	else
		LootLink_ResultsFrame_TitleText:SetTextColor(1.0, 1.0, 1.0);
		
		if (LootLink_ToggleButton_ButtonText) then
			LootLink_ToggleButton_ButtonText:SetTextColor(0.0, 0.8, 0.8);
		end
	end
end




function LootLink:Results_ShowRecentlyAddedItemsTooltip(parent, showHelpText)
	local tooltip = self.RESULTS_TOOLTIP;
	
	local addLine = tooltip.AddLine;
	
	local newItems = self.RecentlyAddedItems;
	
	local numItems = self.MAX_RECENTLY_ADDED_ITEMS;
	local none = NONE;
	
	
	tooltip:SetOwner(parent, "ANCHOR_RIGHT");
	
	if (showHelpText) then
		addLine(tooltip, self.LOCALIZATION.TITLE_TOOLTIP_HELP, 1.0, 0.82, 0.0, 1.0, true);
	end
	
	addLine(tooltip, self.LOCALIZATION.RECENTLY_ADDED_ITEMS_TOOLTIP_TITLE:format(numItems), 1.0, 0.82, 0.0);
	
	
	for index = 1, numItems do
    --SIZ
    if newItems[index] then
      addLine(tooltip, (index..": "..(newItems[index] or none)), 1.0, 0.82, 0.0);
    end
	end
	
	
	LootLink_ResultsTooltip_ItemIcon:SetTexture("");
	
	tooltip:Show();
end




function LootLink:Results_ItemButton_OnEnter(button)
	self.GLOBAL_ENVIRONMENT[button:GetName().."Highlight"]:Show();
	
	local tooltip = self.RESULTS_TOOLTIP;
	tooltip:SetOwner(button, "ANCHOR_NONE");
	tooltip:SetPoint("TOPLEFT", self.RESULTS_FRAME, "TOPRIGHT", -30, -10);
	
	
	local itemTable = button.ItemTable;
	local suffixID = (button.SuffixID or 0);
	local instanceID = (button.InstanceID or 0);
	
	
	local itemName, _, _, _, _, _, _, _, _, texture = GetItemInfo(itemTable[-1]);
	
	if (itemName) then
		local oldName; if (suffixID == 0) then oldName = itemTable[1]; end
		local color = itemTable[2];
		
		-- Reparse the item for the hell of it.
		self:ItemTables_Populate(itemTable, suffixID, instanceID);
		
		-- Update the button's displayed data if the above parsing changed the relevent information.
		if (suffixID == 0 and itemTable[1] ~= oldName) then
			if (itemTable[-3]) then
				button:SetText(itemName.." ("..itemTable[-3]..")");
			else
				button:SetText(itemName);
			end
		end
		
		if (itemTable[2] ~= color) then
			local colorTable = ITEM_QUALITY_COLORS[itemTable[2]];
			button:SetTextColor(colorTable.r, colorTable.g, colorTable.b);
			button:SetHighlightTextColor(colorTable.r, colorTable.g, colorTable.b);
		end
		
		tooltip:SetHyperlink("item:"..itemTable[-1]..":0:0:0:0:0:"..suffixID..":"..instanceID);
		
	else
		self:Results_PopulateFakeItemTooltip(tooltip, itemTable, suffixID, instanceID);
	end
	
	
	LootLink_ResultsTooltip_ItemIcon:SetTexture(texture or itemTable[13] or "Interface\\Icons\\INV_Misc_QuestionMark");
	
	
	self:Tooltips_AddAllApplicableInfo(("item:"..itemTable[-1]..":0:0:0:0:0:"..suffixID..":"..instanceID), tooltip, 1);
	
	
	if (EnhTooltip ~= nil) then
		-- Defined as: TooltipCall(frame,name,link,quality,count,price,forcePopup,hyperlink)
		
		EnhTooltip.TooltipCall(
		  tooltip, -- Tooltip
		  (button.ItemTable[1]..((suffixID ~= 0 and (" "..self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]])) or "")), -- Item name
		  self:Matching_GetHyperlinkByItemTableAndSuffixID(itemTable, suffixID, instanceID), -- Item link
		  itemTable[2], -- Item rarity
		  1, -- Number of items in the stack
		  (itemTable[6] ~= false and itemTable[6]) -- Vendor value for item
		);
	end
end




function LootLink:Results_ItemButton_OnLeave(button)
	self.GLOBAL_ENVIRONMENT[button:GetName().."Highlight"]:Hide();
	self.RESULTS_TOOLTIP:Hide();
end




function LootLink:Results_ItemButton_OnClick(button)
	local itemTable = button.ItemTable;
	
	local link = self:Matching_GetHyperlinkByItemTableAndSuffixID(itemTable, button.SuffixID, button.InstanceID);
	
  if IsShiftKeyDown() then
    if ( WIM_EditBoxInFocus ) then
      WIM_EditBoxInFocus:Insert(link);
    elseif ChatFrameEditBox:IsVisible() then
      ChatFrameEditBox:Insert(link);
    end
--	if (IsShiftKeyDown() and ChatFrameEditBox:IsVisible()) then
--		ChatFrameEditBox:Insert(link);
		
	elseif (IsControlKeyDown()) then
		DressUpItemLink(link);
		
	elseif (IsAltKeyDown()) then
		self.ItemToRemove = { itemTable = itemTable, suffixID = button.SuffixID, instanceID = button.InstanceID };
		StaticPopup_Show("LOOTLINK_CONFIRM_REMOVEITEM", link);
		
	elseif (not itemTable[-3] or button.SuffixID) then
		SetItemRef(link);
		
	else
		local status = itemTable[-2];
		local id = (button:GetID() + LootLink_ResultsFrame_ListScrollFrame.offset);
		
		if (status == 0) then -- This is an item with suffixes which is collapsed. Expand it now.
			self:Results_ShowSuffixesForEntryID(id);
		elseif (status == 1) then -- This is an item with suffixes which is expanded. Collapse it now.
			self:Results_HideSuffixesForEntryID(id);
		end
	end
end



function LootLink:Results_ShowSuffixesForEntryID(entryID)	
	local resultsList = self.ResultsList;
	local itemTable = resultsList[entryID];
	
	if (not itemTable or not itemTable[3] or itemTable[-2] ~= 0 or itemTable[3] == {}) then return; end
	
	
	itemTable[-2] = 1; -- Set the indicator to "expanded".
	
	
	local pairs = pairs;
	local type = type;
	local tableinsert = self.TABLE_LIBRARY.insert;
	
	local sortingTable = self:Main_GetReusableTable(1);
	
	
	for suffixID, suffixData in pairs(itemTable[3]) do
		if (suffixID < 0) then
			for instanceID, instanceData in pairs(suffixData) do
				if (instanceData == true or (type(instanceData) == "table" and instanceData[0] == true)) then
					tableinsert(sortingTable, (suffixID.." "..instanceID));
				end
			end
			
		else
			if (suffixData == true or (type(suffixData) == "table" and suffixData[0] == true)) then
				tableinsert(sortingTable, suffixID);
			end
		end
	end
	
	
	self.TABLE_LIBRARY.sort(sortingTable, self.Sorting_SuffixIDByNameComparison_Standard);
	
	
	local insertionIndex = (entryID + 1);
	
	for index, id in ipairs(sortingTable) do
		tableinsert(resultsList, insertionIndex, id);
	end
	
	
	self.ScrollableResultsSize = (self.ScrollableResultsSize + itemTable[-3]);
	
	self:Results_UpdateCurrentPage();
end



function LootLink:Results_HideSuffixesForEntryID(entryID)
	local resultsList = self.ResultsList;
	local itemTable = resultsList[entryID];
	
	if (not itemTable or not itemTable[3] or itemTable[-2] ~= 1 or itemTable[3] == {}) then return; end
	
	
	itemTable[-2] = 0; -- Set the indicator to "collapsed".
	
	
	local tableremove = self.TABLE_LIBRARY.remove;
	
	local removalIndex = (entryID + 1);
	
	for index = 1, itemTable[-3] do
		tableremove(resultsList, removalIndex);
	end
	
	
	self.ScrollableResultsSize = (self.ScrollableResultsSize - itemTable[-3]);
	
	self:Results_UpdateCurrentPage();
end




function LootLink:Results_PopulateFakeItemTooltip(tooltip, itemTable, suffixID, instanceID)
	if (type(itemTable) == "number") then
		itemTable = self.Database[itemTable];
	end
	
	if (not itemTable) then return; end
	
	
	local tooltipName = tooltip:GetName();
	
	
	if (suffixID and suffixID ~= 0) then
		tooltip:SetText(itemTable[1].." "..self.SUFFIX_NAME_KEYS_TO_NAMES[self.SUFFIX_IDS_TO_NAME_KEYS[suffixID]]);
	else
		tooltip:SetText(itemTable[1]);
	end
	
	
	local colorTable = ITEM_QUALITY_COLORS[itemTable[2]];
	
	if (colorTable) then
		self.GLOBAL_ENVIRONMENT[tooltipName.."TextLeft1"]:SetTextColor(colorTable.r, colorTable.g, colorTable.b);
	else
		self.GLOBAL_ENVIRONMENT[tooltipName.."TextLeft1"]:SetTextColor(0.0, 0.8, 0.8);
	end
	
	
	local numLinesToSkip = 0;
	
--[[	
	local bindType = itemTable[4];
	if (bindType == 1) then
		tooltip:AddLine(ITEM_BIND_ON_EQUIP, 1.0, 1.0, 1.0);
		numLinesToSkip = (numLinesToSkip + 1);
	elseif (bindType == 2) then
		tooltip:AddLine(ITEM_BIND_ON_PICKUP, 1.0, 1.0, 1.0);
		numLinesToSkip = (numLinesToSkip + 1);
	elseif (bindType == 3) then
		tooltip:AddLine(ITEM_BIND_ON_USE, 1.0, 1.0, 1.0);
		numLinesToSkip = (numLinesToSkip + 1);
	elseif (bindType == 4) then
		tooltip:AddLine(ITEM_BIND_QUEST, 1.0, 1.0, 1.0);
		numLinesToSkip = (numLinesToSkip + 1);
	end
	
	
	if (itemTable[11]) then
    DEFAULT_CHAT_FRAME:AddMessage("Unique: "..ITEM_UNIQUE)
		tooltip:AddLine(ITEM_UNIQUE, 1.0, 1.0, 1.0);
		numLinesToSkip = (numLinesToSkip + 1);
	end
	
	
	local location = itemTable[7];
	local itemType = itemTable[8];
	local itemSubtype = itemTable[9];
	
	if (location) then
		-- Special cases...
		if (location == 20) then -- Shield, but should display as offhand.
			tooltip:AddLine(INVTYPE_WEAPONOFFHAND, 1.0, 1.0, 1.0);
			numLinesToSkip = (numLinesToSkip + 1);
			
		else
			local locationsTable = self.EQUIP_LOCATIONS;
			
			local index = 1;
			local categoryTable, subIndex, numEntries;
			
			while (index <= 3) do
				categoryTable = locationsTable[index].subList;
				numEntries = #categoryTable;
				subIndex = 1;
				
				while (subIndex <= numEntries) do
					if (categoryTable[subIndex].value == location) then
						tooltip:AddLine(categoryTable[subIndex].name, 1.0, 1.0, 1.0);
						numLinesToSkip = (numLinesToSkip + 1);
						
						if (index == 3) then
							itemType = nil;
						end
						
						subIndex = (numEntries + 1);
						index = 4;
						
					else
						subIndex = (subIndex + 1);
					end
				end
				
				index = (index + 1);
			end
		end
	end
	
	
	-- Set up TooltipTextRight on the proper line, and as the proper type.
	if (itemType and itemSubtype and itemType ~= self.LOCALIZATION.PARSE_TYPE_GEMS) then
    DEFAULT_CHAT_FRAME:AddMessage("Doing stuff for itemType")
		local next = next;
		
		local typesAndSubtypes = self.TYPES_AND_SUBTYPES;
		
		local prefix = (tooltipName.."TextRight");
		
		local index, typeTable = next(typesAndSubtypes);
		
		while (index) do
			if (typeTable.value == itemType) then
				
				local subIndex, subTable = next(typeTable.subList);
				
				while (subIndex) do
					if (subTable.value == itemSubtype) then
						
						if (location) then
							local tooltipLine = self.GLOBAL_ENVIRONMENT[prefix..(numLinesToSkip + 1)];
							tooltipLine:SetText(subTable.tooltipName or subTable.parseName or subTable.displayName);
							tooltipLine:SetTextColor(1.0, 1.0, 1.0);
							tooltipLine:Show();
							
						else
							tooltip:AddLine(subTable.tooltipName or subTable.parseName or subTable.displayName, 1.0, 1.0, 1.0);
							numLinesToSkip = (numLinesToSkip + 1);
						end
						
						subIndex = nil;
						
					else
						subIndex, subTable = next(typeTable.subList, subIndex);
					end
				end
				
				index = nil;
				
			else
				index, typeTable = next(typesAndSubtypes, index);
			end
		end
	end
]]--		
	
	
	if (itemTable[5]) then
		local globalEnv = self.GLOBAL_ENVIRONMENT;

		local fullText = itemTable[5];

		local prefix = (tooltipName.."TextRight");

		local textRight, tooltipLine;

		for index = 1, 8 do
			textRight = fullText[-index];
			
			if (textRight) then
				tooltipLine = globalEnv[prefix..index];
				tooltipLine:SetText(textRight);
				tooltipLine:SetTextColor(1.0, 1.0, 1.0);
				tooltipLine:Show();
			else
				globalEnv[prefix..index]:Hide();
			end
		end
	

		local stringlen = self.STRING_LIBRARY.len;
		local stringsub = self.STRING_LIBRARY.sub;
		local stringfind = self.STRING_LIBRARY.find;

		local minLevelPrefix = self.LOCALIZATION.FAKETIP_MINLEVEL;
		local minLevelPrefix_Length = stringlen(minLevelPrefix);

		local triggerOnEquip = ITEM_SPELL_TRIGGER_ONEQUIP;
		local triggerOnEquip_Length = stringlen(triggerOnEquip);
		local triggerOnUse = ITEM_SPELL_TRIGGER_ONUSE;
		local triggerOnUse_Length = stringlen(triggerOnUse);
		local triggerOnProc = ITEM_SPELL_TRIGGER_ONPROC;
		local triggerOnProc_Length = stringlen(triggerOnProc);

		local classesPrefix = self.LOCALIZATION.FAKETIP_CLASSES;
		local classesPrefix_Length = stringlen(classesPrefix);
		local racesPrefix = self.LOCALIZATION.FAKETIP_RACES;
		local racesPrefix_Length = stringlen(racesPrefix);
		local setPrefix = self.LOCALIZATION.FAKETIP_SET;
    local socketBonusPrefix = gsub(ITEM_SOCKET_BONUS,"%%s","")


		for index, piece in ipairs(fullText) do
			if (index <= numLinesToSkip and index ~= 3) then
				-- Do nothing.
--        DEFAULT_CHAT_FRAME:AddMessage("Not adding extra data: "..(piece or ""))


			elseif (piece == false) then
				-- Try to add suffix text now.
				local suffixText;

				if (suffixID and suffixID ~= 0 and itemTable[3] and itemTable[3] ~= {}) then
					suffixText = itemTable[3][suffixID];
					
					if (suffixText and suffixID < 0) then
						suffixText = suffixText[instanceID];
					end
				end

				if (suffixText and suffixText ~= true) then
					for subIndex, suffixEntry in ipairs(suffixText) do
						tooltip:AddLine(suffixEntry, 1.0, 1.0, 1.0, 1.0, true);
					end
				end


			else
				if (stringsub(piece, 1, minLevelPrefix_Length) == minLevelPrefix) then
					local level = piece:match("(%d+)");
					if (UnitLevel("player") >= tonumber(level)) then
						tooltip:AddLine(piece, 1.0, 1.0, 1.0, 1.0, true);
					else
						tooltip:AddLine(piece, 1.0, 0.0, 0.0, 1.0, true);
					end
				elseif (stringsub(piece, 1, triggerOnEquip_Length) == triggerOnEquip
				   or stringsub(piece, 1, triggerOnUse_Length) == triggerOnUse
				   or stringsub(piece, 1, triggerOnProc_Length) == triggerOnProc
				) then
					tooltip:AddLine(piece, 0.0, 1.0, 0.0, 1.0, true);
				elseif (stringsub(piece, 1, classesPrefix_Length) == classesPrefix) then
					if (stringfind(piece, (UnitClass("player")), classesPrefix_Length, true)) then
						tooltip:AddLine(piece, 1.0, 1.0, 1.0, 1.0, true);
					else
						tooltip:AddLine(piece, 1.0, 0.0, 0.0, 1.0, true);
					end
				elseif (stringsub(piece, 1, racesPrefix_Length) == racesPrefix) then
					if (stringfind(piece, UnitRace("player"), racesPrefix_Length, true)) then
						tooltip:AddLine(piece, 1.0, 1.0, 1.0, 1.0, true);
					else
						tooltip:AddLine(piece, 1.0, 0.0, 0.0, 1.0, true);
					end
        --Yellow Quoted text
				elseif (stringsub(piece, 1, 1) == "\"" or stringfind(piece, "%(%d+/%d+%)")) then
					tooltip:AddLine(piece, 1.0, 0.8235, 0.0, 1.0, true);
				elseif (stringsub(piece, 1, 2) == "  ") then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
				elseif (stringfind(piece, setPrefix)) then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
        elseif (piece == EMPTY_SOCKET_RED) then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
					tooltip:AddTexture("Interface\\ItemSocketingFrame\\UI-EmptySocket-Red")
        elseif (piece == EMPTY_SOCKET_YELLOW) then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
					tooltip:AddTexture("Interface\\ItemSocketingFrame\\UI-EmptySocket-Yellow")
        elseif (piece == EMPTY_SOCKET_BLUE) then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
					tooltip:AddTexture("Interface\\ItemSocketingFrame\\UI-EmptySocket-Blue")
        elseif (piece == EMPTY_SOCKET_META) then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
					tooltip:AddTexture("Interface\\ItemSocketingFrame\\UI-EmptySocket-Meta")
        elseif (stringfind(piece, socketBonusPrefix)) then
					tooltip:AddLine(piece, 0.5, 0.5, 0.5, 1.0, true);
				else
					tooltip:AddLine(piece, 1.0, 1.0, 1.0, 1.0, true);
				end
			end
		end
		
		
	else
		if (itemTable[10]) then
			tooltip:AddLine(ITEM_MIN_LEVEL:format(itemTable[10]), 1.0, 1.0, 1.0);
		end
	end
	
	
	tooltip:Show();
end

