

local LootLink = LootLink;




LootLink.SelectedSortID = 1; -- Initially set to sort by name.



do
	local dropdown = LootLink_ResultsFrame_SortTypeDD;
	
	dropdown.LOOTLINK = LootLink;
	LootLink.SORTING_DROPDOWN = dropdown;
	
	UIDropDownMenu_SetWidth(90, dropdown);

	dropdown:EnableMouse(true);
end



LootLink_ResultsFrame_SortTypeDDText:SetJustifyH("CENTER");
LootLink_ResultsFrame_SortTypeDDText:SetText(LootLink.LOCALIZATION.SORT_NAME);




function LootLink.Sorting_BaseIDComparison_Standard(elem1, elem2)
	return (elem1[-1] > elem2[-1]);
end



function LootLink.Sorting_BaseIDComparison_Reversed(elem1, elem2)
	return (elem1[-1] < elem2[-1]);
end




function LootLink.Sorting_NameComparison_Standard(elem1, elem2)
	local name1 = elem1[1];
	local name2 = elem2[1];
	
	
	if (name1 == name2) then
		-- If the names are the same, try to sort by equip level. This is a pretty rare case.
		local level1 = elem1[10];
		local level2 = elem2[10];
		
		if (level1 == level2) then
			-- Levels are identical, so sort by ID. This is a very rare case.
			return LootLink.Sorting_BaseIDComparison_Standard(elem1, elem2);
		else
			return ((not level1) or (level2 and level1 > level2));
		end
	end
	
	
	return (name1 < name2);
end



function LootLink.Sorting_NameComparison_Reversed(elem1, elem2)
	local name1 = elem1[1];
	local name2 = elem2[1];
	
	
	if (name1 == name2) then
		-- If the names are the same, try to sort by equip level. This is a pretty rare case.
		local level1 = elem1[10];
		local level2 = elem2[10];
		
		if (level1 == level2) then
			-- Levels are identical, so sort by ID. This is a very rare case.
			return LootLink.Sorting_BaseIDComparison_Standard(elem1, elem2);
		else
			return ((not level1) or (level2 and level1 < level2));
		end
	end
	
	
	return (name1 > name2);
end




function LootLink.Sorting_RarityComparison_Standard(elem1, elem2)
	if (elem1[2] == elem2[2]) then
		-- Rarities are identical, so sort by name. This is by far the most common case.
		return LootLink.Sorting_NameComparison_Standard(elem1, elem2);
	end
	
	-- Rarities are NOT identical, so sort by rarity.
	return (elem1[2] > elem2[2]);
end



function LootLink.Sorting_RarityComparison_Reversed(elem1, elem2)
	if (elem1[2] == elem2[2]) then
		-- Rarities are identical, so sort by name. This is by far the most common case.
		return LootLink.Sorting_NameComparison_Standard(elem1, elem2);
	end
	
	-- Rarities are NOT identical, so sort by rarity.
	return (elem1[2] < elem2[2]);
end




local tonumber = tonumber;

local LL_SuffixIDsToNameKeys = LootLink.SUFFIX_IDS_TO_NAME_KEYS;
local LL_SuffixNameKeysToNames = LootLink.SUFFIX_NAME_KEYS_TO_NAMES;



function LootLink.Sorting_SuffixIDByNameComparison_Standard(id1, id2)
	return (
	  LL_SuffixNameKeysToNames[LL_SuffixIDsToNameKeys[tonumber(id1) or tonumber(id1:match("^[^%s]+"))]]
	  < LL_SuffixNameKeysToNames[LL_SuffixIDsToNameKeys[tonumber(id2) or tonumber(id2:match("^[^%s]+"))]]
	);
end



function LootLink.Sorting_SuffixIDByNameComparison_Reversed(id1, id2)
	return (
	  LL_SuffixNameKeysToNames[LL_SuffixIDsToNameKeys[tonumber(id1) or tonumber(id1:match("^[^%s]+"))]]
	  > LL_SuffixNameKeysToNames[LL_SuffixIDsToNameKeys[tonumber(id2) or tonumber(id2:match("^[^%s]+"))]]
	);
end




local LL_KeyNum;



function LootLink.Sorting_KeyComparison_Standard(elem1, elem2)
	local v1 = elem2[LL_KeyNum];
	local v2 = elem1[LL_KeyNum];
	
	if (v1 == v2) then
		return LootLink.Sorting_RarityComparison_Standard(elem1, elem2);
	else
		return ((not v1) or (v2 and v1 < v2));
	end
end



function LootLink.Sorting_KeyComparison_Reversed(elem1, elem2)
	local v1 = elem2[LL_KeyNum];
	local v2 = elem1[LL_KeyNum];
	
	if (v1 == v2) then
		return LootLink.Sorting_RarityComparison_Standard(elem1, elem2);
	else
		return ((not v1) or (v2 and v1 > v2));
	end
end




function LootLink:Sorting_SortBySortID(sortID, reversed)
	local sortTable = self.SORTING_LIST[sortID or self.SelectedSortID];
	
	if (not sortTable) then return; end
	
	
	if (reversed) then
		self.SortIsReversed = true;
	else
		self.SortIsReversed = nil;
	end
	
	
	if (sortTable.comparisonKey) then
		LL_KeyNum = sortTable.comparisonKey;
		self.TABLE_LIBRARY.sort(self.ResultsList, ((reversed and self.Sorting_KeyComparison_Reversed) or self.Sorting_KeyComparison_Standard));
	else
		self.TABLE_LIBRARY.sort(self.ResultsList, ((reversed and sortTable.reversed) or sortTable.standard));
	end
end




function LootLink:Sorting_SortTypeDDButton_OnClick()
	local button = this;
	
	if (self.SelectedSortID ~= button.value) then
		self.SelectedSortID = button.value;
		self.SortIsReversed = nil;
		
		LootLink_ResultsFrame_SortTypeDDText:SetText(button:GetText());
		
	else
		if (self.SortIsReversed) then
			self.SortIsReversed = nil;
		else
			self.SortIsReversed = true;
		end
	end
	
	self:Results_FullRebuild();
end



function LootLink:Sorting_SortTypeDD_Initialize()
	local buttonInfo = self:Main_GetReusableTable(1);
	buttonInfo.func = self.Sorting_SortTypeDDButton_OnClick;
	buttonInfo.arg1 = self;
	
	local addButton = UIDropDownMenu_AddButton;
	
	local currentID = self.SelectedSortID;
	
	for index, sortTable in ipairs(self.SORTING_LIST) do
		buttonInfo.text = sortTable.name;
		buttonInfo.value = index;
		buttonInfo.checked = (index == currentID);
		addButton(buttonInfo);
	end
end


LootLink.SORTING_DROPDOWN.initialize = (function(level) LootLink:Sorting_SortTypeDD_Initialize(); end);




LootLink.SORTING_DROPDOWN:SetScript("OnMouseUp",
  function(self, button)
	local lootlink = self.LOOTLINK;
	
    if (lootlink.SortIsReversed) then
		lootlink.SortIsReversed = nil;
	else
		lootlink.SortIsReversed = true;
	end
	
	lootlink:Results_FullRebuild();
  end
);




do
	local sortingList = LootLink.SORTING_LIST;
	
	local currentEntry;
	
	
	currentEntry = sortingList[8];
	
	currentEntry.standard = LootLink.Sorting_BaseIDComparison_Standard;
	currentEntry.reversed = LootLink.Sorting_BaseIDComparison_Reversed;
	
	
	currentEntry = sortingList[1];
	
	currentEntry.standard = LootLink.Sorting_NameComparison_Standard;
	currentEntry.reversed = LootLink.Sorting_NameComparison_Reversed;
	
	
	currentEntry = sortingList[2];
	
	currentEntry.standard = LootLink.Sorting_RarityComparison_Standard;
	currentEntry.reversed = LootLink.Sorting_RarityComparison_Reversed;
end

