

function LootLink:DataSharing_Initialize()




local LootLink = self;




LootLink.DATASHARING_CHANNEL_NAME = ("lootlinkdatasharing");
LeaveChannelByName(LootLink.DATASHARING_CHANNEL_NAME);



do
	local scripts = CreateFrame("Frame", "LootLink_DataSharing_BroadcastFrame", WorldFrame);
	
	scripts:Hide();
	
	scripts.Elapsed = 0.0;
	
	scripts.LOOTLINK = LootLink;
	self.DATASHARING_BROADCAST_FRAME = scripts;
end



LootLink.DATASHARING_PARSING_MAIN_PATTERN = ("(%d+)=([^>]*)>?");

LootLink.DATASHARING_PARSING_SUB_PATTERN = ("(%d+):([^@]*)@");


LootLink.DATASHARING_FORMATTING_BASIC_PATTERN = ("%s>%d=1:%s@2:%d@");

LootLink.DATASHARING_FORMATTING_SPECIFIC_PATTERN = ("%s%d:%s@");



if (LootLink.Original_ChatFrame_MessageEventHandler == nil) then
	LootLink.Original_ChatFrame_MessageEventHandler = ChatFrame_MessageEventHandler;
	ChatFrame_MessageEventHandler = LootLink.ReplaceHook_ChatFrame_MessageEventHandler;
end




function LootLink:DataSharing_ParseMessage(message, sender)
	if (message == self.LastDataMessageParsed or sender == self.PLAYER_NAME) then return 0, 0; end
	
	self.LastDataMessageParsed = message;
	
	
	local numParsed, numNew;
	
	if (self.TrustedDataSendersForCurrentRealm[sender:lower()]) then
		numParsed, numNew = self:DataSharing_ParseMessage_Trusted(message);
	else
		numParsed, numNew = self:DataSharing_ParseMessage_NonTrusted(message);
	end
	
	
	if (numNew > 0) then
		self.DatabaseSize = (self.DatabaseSize + numNew);
		self:Results_UpdateTitle(true);
	end
	
	
	return numParsed, numNew;
end



function LootLink:DataSharing_ParseMessage_Trusted(message)
	local stringgmatch = self.STRING_LIBRARY.gmatch;
	local tonumber = tonumber;
	
	local database = self.Database;
	
	local subPattern = self.DATASHARING_PARSING_SUB_PATTERN;
	local numParsed = 0;
	local numNew = 0;
	
	local currentTable;
	local tempItem = self:Main_GetReusableTable(1);
	local isNewItem;
	
	
	for baseID, details in stringgmatch(message, self.DATASHARING_PARSING_MAIN_PATTERN) do
		baseID = tonumber(baseID);
		
		currentTable = database[baseID];
		
		if (not currentTable) then
			tempItem[baseID] = self:Main_GetReusableTable(2);
			isNewItem = true;
		else
			isNewItem = nil;
		end
		
		
		if (isNewItem) then
			for key, value in stringgmatch(details, subPattern) do
				tempItem[tonumber(key)] = (tonumber(value) or value);
			end
			
			if (tempItem[1] and tempItem[2]) then
				currentTable = self:ItemTables_AddBaseItem(baseID, tempItem[1], tempItem[2]);
				numNew = (numNew + 1);
			end
			
			
		else
			for key, value in stringgmatch(details, subPattern) do
				key = tonumber(key);
				value = (tonumber(value) or value);
				
				if (key == 6) then
					currentTable[6] = value;
					
				elseif (currentTable[key] == nil) then
					currentTable[key] = value;
				end
			end
		end
		
		
		-- Special check for vendor price data.
		if (currentTable and currentTable[6] == 0) then currentTable[6] = false; end
		
		
		numParsed = (numParsed + 1);
	end
	
	
	return numParsed, numNew;
end



function LootLink:DataSharing_ParseMessage_NonTrusted(message)
	local stringgmatch = self.STRING_LIBRARY.gmatch;
	local tonumber = tonumber;
	
	local database = self.Database;
	
	local subPattern = self.DATASHARING_PARSING_SUB_PATTERN;
	local numParsed = 0;
	
	local currentTable;
	
	
	for baseID, details in stringgmatch(message, self.DATASHARING_PARSING_MAIN_PATTERN) do
		baseID = tonumber(baseID);
		
		currentTable = database[baseID];
		
		
		if (currentTable) then
			for key, value in stringgmatch(details, subPattern) do
				key = tonumber(key);
				
				if (currentTable[key] == nil) then
					currentTable[key] = (tonumber(value) or value);
				end
			end
			
			-- Special check for vendor price data.
			if (currentTable[6] == 0) then currentTable[6] = false; end
		end
		
		
		numParsed = (numParsed + 1);
	end
	
	
	return numParsed, 0;
end




function LootLink:DataSharing_GetMessageToSend(lastKey, maxLength)
	if (not lastKey) then lastKey = self.LastBroadcastKey; end
	
	if (not maxLength) then maxLength = 255; end
	
	
	local next = next;
	local stringlen = self.STRING_LIBRARY.len;
	local stringformat = self.STRING_LIBRARY.format;
	
	local database = self.Database;
	
	local basicPattern = self.DATASHARING_FORMATTING_BASIC_PATTERN;
	local specificPattern = self.DATASHARING_FORMATTING_SPECIFIC_PATTERN;
	
	local testMessage = ("");
	local messageToSend;
	
	
	local baseID, itemTable = next(database, lastKey);
	
	while (stringlen(testMessage) <= maxLength and baseID) do
		messageToSend = testMessage;
		
		testMessage = stringformat(
		  basicPattern,
		  testMessage,
		  baseID,
		  itemTable[1],
		  itemTable[2]
		);
		
		if (itemTable[6]) then
			testMessage = stringformat(
			  specificPattern,
			  testMessage,
			  6,
			  ((itemTable[6] == false and 0) or itemTable[6])
			);
		end
		
		baseID, itemTable = next(database, baseID);
	end
	
	
	return messageToSend, baseID;
end




function LootLink:DataSharing_SendNextMessage()
	local messageToSend;
	
	messageToSend, self.LastBroadcastKey = self:DataSharing_GetMessageToSend(self.LastBroadcastKey, (254 - #self.DATASHARING_CHANNEL_NAME));
	
	
	local sendAddonMessage = SendAddonMessage;
	
	local dataChannel = self.DATASHARING_CHANNEL_NAME;
	
	
	for chatType in pairs(self.Config.DataSharingBroadcastChannels) do
		if (chatType == "CHANNEL") then
			local clearAFK = GetCVar("autoClearAFK");
			SetCVar("autoClearAFK", 0);
			
			local channelNumber = GetChannelName(dataChannel);
			if (channelNumber == 0) then
				channelNumber = JoinChannelByName(dataChannel);
			end
			if (channelNumber ~= 0) then
				SendChatMessage(messageToSend, "CHANNEL", nil, channelNumber);
			end
			
			SetCVar("autoClearAFK", clearAFK);
			
		elseif (chatType ~= "GUILD" or IsInGuild()) then
			sendAddonMessage(dataChannel, messageToSend, chatType);
		end
	end
end




LootLink.DATASHARING_BROADCAST_FRAME:SetScript("OnUpdate", function(self, elapsed)
	self.Elapsed = (self.Elapsed + elapsed);
	
	if (self.Elapsed > 15.0) then
		self.Elapsed = 0.0;
		
		self.LOOTLINK:DataSharing_SendNextMessage();
	end
end);




end

